@@ -14,7 +14,7 @@ import sys
 import time
 import uuid
 import json
-from typing import List
+from typing import List, Callable
 
 import numpy as np
 from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session,get_backend, \
@@ -22,21 +22,24 @@ from trident.backend.common import to_list, addindent, get_time_suffix, format_t
 from trident.backend.tensorspec import *
 from trident.data.image_common import *
 from trident.callbacks import LambdaCallback, UnfreezeModelCallback
+from trident.loggers.history import HistoryBase
 
 _session = get_session()
-_backend = get_backend()
-if _backend == 'pytorch':
+working_directory=_session.working_directory
+
+
+if get_backend() == 'pytorch':
     from trident.backend.pytorch_backend import *
     from trident.backend.pytorch_ops import *
 
-elif _backend == 'tensorflow':
+elif get_backend() == 'tensorflow':
     from trident.backend.tensorflow_backend import *
     from trident.backend.tensorflow_ops import *
 
 
 
 
-__all__ = ['progress_bar','ModelBase','HistoryBase']
+__all__ = ['progress_bar','ModelBase']
 
 
 
@@ -55,11 +58,12 @@ class ModelBase(object):
         if isinstance(inputs,tuple) and isinstance(inputs[0],int):
             input_shape,inputs=inputs,input_shape
         self.batch_index = 0
-        self.filter_index = 1
+        self.filter_index =1 if get_backend() == 'pytorch' else -1
         self.inputs = OrderedDict()
         self._outputs = OrderedDict()
         self._targets = OrderedDict()
         self._model = None
+        self.output_fn:Callable=None
 
         self.lr_scheduler = None
         self._losses = OrderedDict()
@@ -70,7 +74,8 @@ class ModelBase(object):
         self._constraints = OrderedDict()
 
 
-        self.preprocess_flow = []
+        self._preprocess_flow = []
+        self.input_spec=None
 
         self.current_save_path=None
 
@@ -84,15 +89,15 @@ class ModelBase(object):
         self.grad_clipping_threshold = None
         self.use_output_as_loss = False
         self.training_context = {
-                                 'losses': HistoryBase('losses'),  # loss_wrapper
-                                 'metrics': HistoryBase('metrics'),  # loss_wrapper
-                                 'epoch_losses': HistoryBase('epoch_losses'),  # loss_wrapper
-                                 'epoch_metrics': HistoryBase('epoch_metrics'),  # loss_wrapper
-                                 'grads_state': OrderedDict(),  # loss_wrapper
-                                 'tmp_losses': HistoryBase('tmp_losses'), # loss_wrapper
-                                 'tmp_metrics': HistoryBase('tmp_metrics'),  # loss_wrapper
+                                 'losses': HistoryBase('losses'),
+                                 'metrics': HistoryBase('metrics'),
+                                 'epoch_losses': HistoryBase('epoch_losses'),
+                                 'epoch_metrics': HistoryBase('epoch_metrics'),
+                                 'grads_state': OrderedDict(),
+                                 'tmp_losses': HistoryBase('tmp_losses'),
+                                 'tmp_metrics': HistoryBase('tmp_metrics'),
                                  'out_sample_metrics': HistoryBase('out_sample_metrics'),
-                                'print_progress_frequency': 10,
+                                 'print_progress_frequency': 10,
                                 'print_progress_unit': 'batch',
                                 'optimizer':None,
                                  'warmup':0,
@@ -112,15 +117,15 @@ class ModelBase(object):
                                  'best_model': None,  # current model
                                 'loss_history': None, 'metric_history': None, 'base_lr': None,  # current loss
                                 'current_lr': None,  # current loss
-                                'save_path': None, 'is_collect_data': True, 'callbacks': [],
+                                'save_path': os.path.join(working_directory,'Models'), 'is_collect_data': True, 'callbacks': [],
                                  'stop_update': 0, 'retain_graph': False,
                                  'skip_generate_output':False}
         if name is not None:
             self.name=name
 
 
-
-        self._initial_graph(inputs, input_shape,output)
+        if output is not None and (inputs is not None or input_shape is not None):
+            self._initial_graph(inputs, input_shape,output)
 
     @property
     def model(self):
@@ -259,6 +264,15 @@ class ModelBase(object):
         elif len(self.signature) != len(arg_names):
             raise ValueError('data deed and arg_names should be the same length')
 
+    @property
+    def preprocess_flow(self):
+        return self._preprocess_flow
+
+    @preprocess_flow.setter
+    def preprocess_flow(self,value):
+        self._preprocess_flow=value
+        if isinstance(self.input_spec,TensorSpec):
+            self.input_spec=None
 
     @property
     def reverse_preprocess_flow(self):
@@ -272,15 +286,21 @@ class ModelBase(object):
         return return_list
 
     def data_preprocess(self, img_data):
+
         if img_data.ndim==4:
-            return to_tensor(to_numpy([self.data_preprocess(im) for im in img_data]))
+            return to_numpy([self.data_preprocess(im) for im in img_data])
         if len(self.preprocess_flow) == 0:
-            return image_backend_adaption(img_data)
+            return np.expand_dims(image_backend_adaption(img_data),0)
         if isinstance(img_data, np.ndarray):
             for fc in self.preprocess_flow:
-                if not fc.__qualname__.startswith('random_') or  'crop' in fc.__qualname__  or  'rescale' in fc.__qualname__  or  (fc.__qualname__.startswith('random_') and random.randint(0,10)%2==0):
+                if self._model is not None and self.signature is not None and len(self.signature) > 1 and self.input_spec is not None:
+                    img_data = fc(img_data,spec=self.input_spec)
+                else:
                     img_data = fc(img_data)
-            img_data = to_tensor(image_backend_adaption(img_data)).unsqueeze(0)
+            img_data = np.expand_dims(image_backend_adaption(img_data),0)
+            if self.input_spec is None :
+                self.input_spec= TensorSpec(shape=tensor_to_shape(to_tensor(img_data),need_exclude_batch_axis=False), object_type=ObjectType.rgb, name='input')
+
             return img_data
         else:
             return img_data
@@ -360,29 +380,32 @@ class ModelBase(object):
         raise AttributeError("'{}' object has no attribute '{}'".format(type(self).__name__, name))
 
     def __setattr__(self, name, value):
-        if name=='signature' or name=='_signature':
-            _model = self.__dict__['_model']
-            if _model is not None and isinstance(_model ,Layer):
-                object.__setattr__(_model, "_" + 'signature', value)
-        if 'training_context' in self.__dict__ and name in self.__dict__['training_context']:
-            self.__dict__['training_context'][name]=value
-        elif '_model' in self.__dict__ and self.__dict__['_model']  :
-            _model = self.__dict__['_model']
-            if _model is not None and name in _model.__dict__['_parameters']:
-                _model.__dict__['_parameters'][name]=value
-            elif _model is not None and name in _model.__dict__['_buffers']:
-                 _model.__dict__['_buffers'][name]=value
-            elif _model is not None and name in _model.__dict__['_modules']:
-                _model.__dict__['_modules'][name]=value
+        if name in ['_input_shape','_output_shape']:
+            pass
+        else:
+            if name=='signature' or name=='_signature':
+                _model = self.__dict__['_model']
+                if _model is not None and isinstance(_model ,Layer):
+                    object.__setattr__(_model, "_" + 'signature', value)
+            if 'training_context' in self.__dict__ and name in self.__dict__['training_context']:
+                self.__dict__['training_context'][name]=value
+            elif '_model' in self.__dict__ and self.__dict__['_model']  :
+                _model = self.__dict__['_model']
+                if _model is not None and name in _model.__dict__['_parameters']:
+                    _model.__dict__['_parameters'][name]=value
+                elif _model is not None and name in _model.__dict__['_buffers']:
+                     _model.__dict__['_buffers'][name]=value
+                elif _model is not None and name in _model.__dict__['_modules']:
+                    _model.__dict__['_modules'][name]=value
 
-            elif _model is not None and name in _model.__dict__:
-                object.__setattr__(self.__dict__['_model'], name, value)
-            elif _model is not None and "_"+name in _model.__dict__:
-                object.__setattr__(self.__dict__['_model'], "_"+name, value)
+                elif _model is not None and name in _model.__dict__:
+                    object.__setattr__(self.__dict__['_model'], name, value)
+                elif _model is not None and "_"+name in _model.__dict__:
+                    object.__setattr__(self.__dict__['_model'], "_"+name, value)
+                else:
+                    object.__setattr__(self, name, value)
             else:
                 object.__setattr__(self, name, value)
-        else:
-            object.__setattr__(self, name, value)
 
     def __call__(self, *input, **kwargs):
         return self._model(*input, **kwargs)
@@ -442,6 +465,10 @@ class ModelBase(object):
                """
         return self
 
+    def adjust_learning_rate_scheduling(self, index: int, unit='batch', new_value:float=None):
+        return self
+
+
     def reset_training_context(self):
         self.training_context = {
             'losses': HistoryBase('losses'),  # loss_wrapper
@@ -472,7 +499,7 @@ class ModelBase(object):
             'best_model': None,  # current model
             'loss_history': None, 'metric_history': None, 'base_lr': None,  # current loss
             'current_lr': None,  # current loss
-            'save_path': None, 'is_collect_data': True, 'callbacks': [],
+            'save_path': os.path.join(working_directory,'Models'), 'is_collect_data': True, 'callbacks': [],
             'stop_update': 0, 'retain_graph': False,
             'skip_generate_output': False}
 
@@ -515,7 +542,7 @@ class ModelBase(object):
 
     def do_on_data_received(self, train_data, test_data):
 
-        return train_data, test_data
+        return self
 
     def do_preparation_for_loss(self):
         pass
@@ -694,12 +721,7 @@ class ModelBase(object):
                     self.do_on_training_start()
                     # epoch is not the logical inteval for us to control the flow
                     self.training_context['steps']=0
-                    self.training_context['tmp_losses'] = HistoryBase(name='tmp_losses')
-                    self.training_context['tmp_metrics'] =HistoryBase(name='tmp_metrics')
-                    self.training_context['out_sample_metrics'] = HistoryBase(name='out_sample_metrics')
-                    self.training_context['losses'] = HistoryBase(name='losses')
-                    self.training_context['losses'].regist('total_losses')
-                    self.training_context['metrics'] = HistoryBase(name='metrics')
+
                     self.training_context['grads_state'] = OrderedDict()
                     self.training_context['grads_state']['first_layer'] = []
                     self.training_context['grads_state']['last_layer'] = []
@@ -729,27 +751,30 @@ class ModelBase(object):
 
             if  'skip_generate_output' not in self.training_context or self.training_context['skip_generate_output']==False:
                 try:
-                    output = try_map_args_and_call(self._model, train_data, self.training_context['data_feed'])
-                    if isinstance(output, (list, tuple)):
-                        for i in range(len(output)):
-                            train_data[self.outputs.key_list[i]] = output[i]
-                    elif isinstance(output, (OrderedDict)):
-                        for k,v in output.items():
-                            train_data[k] = v
-                    elif 'tensor' in output.__class__.__name__.lower():
-                        train_data[self.outputs.key_list[0]] = output
-                        if self.use_output_as_loss==True:
-
-                            this_loss=output.sum()
-                            self.training_context['losses'].collect(self.outputs.key_list[0],self.training_context['steps'],this_loss)
-                            self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+                    if self.output_fn is not None and callable(self.output_fn):
+                        self.output_fn()
                     else:
-                        train_data[self.outputs.key_list[0]] = output
-                        if self.use_output_as_loss==True:
+                        output = try_map_args_and_call(self._model, self.train_data, self.training_context['data_feed'])
+                        if isinstance(output, (list, tuple)):
+                            for i in range(len(output)):
+                                self.train_data[self.outputs.key_list[i]] = output[i]
+                        elif isinstance(output, (OrderedDict)):
+                            for k,v in output.items():
+                                self.train_data[k] = v
+                        elif 'tensor' in output.__class__.__name__.lower():
+                            self.train_data[self.outputs.key_list[0]] = output
+                            if self.use_output_as_loss==True:
 
-                            this_loss=output.sum()
-                            self.training_context['losses'].collect(self.outputs.key_list[0], self.training_context['steps'], this_loss)
-                            self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+                                this_loss=output.sum()
+                                self.training_context['losses'].collect(self.outputs.key_list[0],self.training_context['steps'],this_loss)
+                                self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
+                        else:
+                            self.train_data[self.outputs.key_list[0]] = output
+                            if self.use_output_as_loss==True:
+
+                                this_loss=output.sum()
+                                self.training_context['losses'].collect(self.outputs.key_list[0], self.training_context['steps'], this_loss)
+                                self.training_context['current_loss'] = self.training_context['current_loss'] + this_loss
                 except Exception as e:
                     print(e)
                     PrintException()
@@ -775,7 +800,7 @@ class ModelBase(object):
                         if k in self.loss_weights:
                             loss_weight = self.loss_weights[k]
                         loss_weight=to_tensor(loss_weight,'float32')
-                        this_loss = loss_weight*try_map_args_and_call(v, train_data, self.training_context['data_feed']) # v.forward(output, target) if hasattr(v, 'forward') else v(
+                        this_loss = loss_weight*try_map_args_and_call(v, self.train_data, self.training_context['data_feed']) # v.forward(output, target) if hasattr(v, 'forward') else v(
 
                         if isinstance(this_loss, tuple):
                             overall_loss =to_tensor(0.0,requires_grad=True)
@@ -814,7 +839,7 @@ class ModelBase(object):
                         this_loss = v(self._model) if self.training_context['stop_update'] < 1 else to_tensor(0.0,requires_grad=True)
                     elif 'output' in v.signature.inputs:
 
-                        this_loss = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if self.training_context['stop_update'] < 1 else to_tensor(0.0)
+                        this_loss = try_map_args_and_call(v, self.train_data, self.training_context['data_feed']) if self.training_context['stop_update'] < 1 else to_tensor(0.0)
                     if not any_abnormal_number(this_loss):
                         # a leaf Variable that requires grad connotused in an in-place operation.
                         self.training_context['current_loss'] =self.training_context['current_loss'] + this_loss  # self.training_context[
@@ -855,15 +880,15 @@ class ModelBase(object):
 
 
 
-                if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0 and  self.training_context['stop_update']<1 :
-                    tmp_output = try_map_args_and_call(self._model, test_data, self.training_context['data_feed'])
+                if is_out_sample_evaluation==True and self.test_data is not None and len(self.test_data) > 0 and  self.training_context['stop_update']<1 :
+                    tmp_output = try_map_args_and_call(self._model, self.test_data, self.training_context['data_feed'])
                     if isinstance(tmp_output, (list, tuple)):
                         for i in range(len(tmp_output)):
-                            test_data[self.outputs.key_list[i]] = tmp_output[i]
+                            self.test_data[self.outputs.key_list[i]] = tmp_output[i]
                     elif 'tensor' in tmp_output.__class__.__name__.lower():
-                        test_data[self.outputs.key_list[0]] = tmp_output
+                        self.test_data[self.outputs.key_list[0]] = tmp_output
                     else:
-                        test_data[self.outputs.key_list[0]] = tmp_output
+                        self.test_data[self.outputs.key_list[0]] = tmp_output
 
 
 
@@ -880,12 +905,12 @@ class ModelBase(object):
                         self.training_context['metrics'].regist(k)
                         self.training_context['tmp_metrics'].regist(k)
 
-                    this_metric = try_map_args_and_call(v, train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else to_tensor(0)
+                    this_metric = try_map_args_and_call(v, self.train_data, self.training_context['data_feed']) if  self.training_context['stop_update']<1 else to_tensor(0)
                     self.training_context['tmp_metrics'].collect(k, self.training_context['steps'], this_metric)
 
 
-                    if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0 and collect_history!=False :
-                        this_out_metric = try_map_args_and_call(v, test_data , self.training_context['data_feed'])
+                    if is_out_sample_evaluation==True and self.test_data is not None and len(self.test_data) > 0 and collect_history!=False :
+                        this_out_metric = try_map_args_and_call(v, self.test_data , self.training_context['data_feed'])
                         self.training_context['out_sample_metrics'].collect(k, self.training_context['steps'], this_out_metric)
 
                 # ON_EVALUATION_END
@@ -923,7 +948,7 @@ class ModelBase(object):
                 else:
                     self.training_context['print_batch_progress_frequency'] += 1
 
-                if is_out_sample_evaluation==True and test_data is not None and len(test_data) > 0:
+                if is_out_sample_evaluation==True and self.test_data is not None and len(self.test_data) > 0:
                     verbose=[]
                     for k in self.training_context['out_sample_metrics'].get_keys():
                         test_steps, test_values = self.training_context['out_sample_metrics'].get_series(k)
@@ -937,7 +962,7 @@ class ModelBase(object):
                             format_string = '.3e'
                         verbose.append('{0}: {1:<8{2}}'.format(k, metric_value, format_string))
                     print(self.training_context['model_name'] + ': out-of-sample evaluation: ',','.join(verbose))
-
+            self.training_context['steps'] += 1
             if self.training_context['current_batch'] == self.training_context['total_batch'] - 1:
                 self.do_on_epoch_end()
                 batch_steps,batch_values=self.training_context['losses'].get_series('total_losses')
@@ -1009,8 +1034,8 @@ class ModelBase(object):
         self.with_callbacks(new_callbacks)
         return self
 
-    def unfreeze_model_scheduling(self, frequency: int, unit='epoch', slice_from=0, slice_to=None):
-        self.callbacks.append(UnfreezeModelCallback(frequency, unit, slice_from, slice_to))
+    def unfreeze_model_scheduling(self, frequency: int, unit='epoch', slice_from=None, slice_to=None, module_name=None):
+        self.callbacks.append(UnfreezeModelCallback(frequency, unit, slice_from, slice_to,module_name=module_name))
         return self
 
     def cpu(self):
@@ -1065,57 +1090,3 @@ def progress_bar(step_time,current, total, msg=None, name=''):
     sys.stdout.flush()  # # Go back to the center of the bar.  # for i in range(term_width-int(TOTAL_BAR_LENGTH/2)+2):  #     sys.stdout.write('\b')  # sys.stdout.write(' %d/%d ' % (current+1, total))  # if current < total-1:  #     sys.stdout.write('\r')  # else:  #     sys.stdout.write('\n')  # sys.stdout.flush()
 
 
-
-class HistoryBase(OrderedDict):
-    def __init__(self, name='', *args, **kwargs):
-        super().__init__(*args, **kwargs)
-        self.name=name
-
-    def regist(self,data_name:str):
-        if data_name not in self:
-            self[data_name]=[]
-
-    def collect(self,data_name:str,step:int,value:(float,Tensor)):
-        if data_name not in self:
-            self.regist(data_name)
-        if is_tensor(value):
-            value=value.clone().cpu().detach()
-            if ndim(value)==0:
-                value=value.item()
-            elif ndim(value)==1 and len(value)==1:
-                value = value[0].item()
-            else:
-                value=value.mean().item()
-            self[data_name].append((step, value))
-        else:
-            self[data_name].append((step, value))
-
-
-    def reset(self):
-        for i in range(len(self)):
-            self.value_list[i]=[]
-    def get_keys(self):
-        return self.key_list
-
-    def get_series(self,data_name):
-        if data_name in self:
-            steps,values=zip(*self[data_name].copy())
-            return list(steps),list(values)
-        else:
-            raise ValueError('{0} is not in this History.'.format(data_name))
-
-    def get_last(self,data_name):
-        if data_name in self:
-            return self[data_name][-1]
-        else:
-            raise ValueError('{0} is not in this History.'.format(data_name))
-
-    def get_best(self,data_name,is_larger_better=True):
-            if data_name in self:
-                steps,values=zip(*self[data_name].copy())
-                if is_larger_better:
-                    return builtins.max(values)
-                else:
-                    return builtins.min(values)
-            else:
-                raise ValueError('{0} is not in this History.'.format(data_name))
\ No newline at end of file
