@@ -2,18 +2,18 @@ import inspect
 import numbers
 from enum import Enum
 from inspect import signature
-from trident.backend.common import to_list, OrderedDict, Signature, split_path, unpack_singleton, get_session
+from trident.backend.common import to_list, OrderedDict, Signature, split_path, unpack_singleton, get_session,get_backend,TensorShape
 from typing import Optional, Union, overload
 import numpy as np
 
 __all__ = ['TensorSpec', 'ObjectType', 'assert_input_compatibility', 'assert_spec_compatibility', 'get_python_function_arguments', 'get_signature', 'ExpectDataType']
 
-_session = get_session()
-_backend = _session.backend
-if _backend == 'pytorch':
-    from trident.backend.pytorch_ops import *
-elif _backend == 'tensorflow':
-    from trident.backend.tensorflow_ops import *
+
+if get_backend()== 'pytorch':
+    from  trident.backend.pytorch_ops import *
+elif get_backend() == 'tensorflow':
+    from trident.backend.tensorflow_ops  import *
+
 
 
 class ObjectType(Enum):
@@ -63,21 +63,25 @@ class TensorSpec(object):
       >>> print(spec.__class__.__name__)
       TensorSpec
       >>> print(spec)
-      TensorSpec(shape=(2, 3, 4), ndim=3)
-
+      TensorSpec(shape=(None, 2, 3, 4), ndim=3)
+      >>> t1=cast(arange(10).reshape(1,2,5),'float16')
+      >>> print(t1.dtype)
+      float16
+      >>> TensorSpec.tensor_to_spec(t1)
+      TensorSpec(dtype=torch.float32, shape=(None, 2, 5), ndim=3)
     """
 
     def __init__(self,
-                 shape=None,
-                 ndim=None,
-                 max_ndim=None,
-                 min_ndim=None,
+                 shape:Union[None,TensorShape]=None,
+                 ndim:Union[None,int]=None,
+                 max_ndim:Union[None,int]=None,
+                 min_ndim:Union[None,int]=None,
                  axes=None,
                  dtype=None,
                  object_type: Optional[ObjectType] = None,
                  is_spatial=False,
                  name=None):
-        self._dtype = dtype if dtype is not None else str2dtype('float32')
+        self._dtype = dtype if dtype is not None else None
         self._shape_tuple = None
         self.object_type = object_type
         if object_type is not None:
@@ -87,20 +91,30 @@ class TensorSpec(object):
                 self.is_spatial = is_spatial
         self._name = name
         if shape is not None:
-            t = to_tensor([-1]).int().to('cpu')
-            if isinstance(shape, (list, tuple)) and all([isinstance(item, numbers.Number) for item in shape]):
+            if isinstance(shape,TensorShape) :
+                self.ndim =shape.ndims
+                self._shape_tuple =tuple(shape.dims)
+                self.shape =shape
+            elif isinstance(shape, (list, tuple)) and all([isinstance(item, numbers.Number) for item in shape]):
                 self.ndim = len(shape)
-                self._shape_tuple = (-1,)+tuple(shape)
-                self.shape = to_tensor(self._shape_tuple).int().to('cpu')
-            elif type(shape) == int or type(shape) == float:
+                self._shape_tuple = (None,)+tuple(shape)
+                self.shape = TensorShape(shape)
+            elif not is_tensor(shape) and  isinstance(shape,numbers.Number) :
                 self.ndim = 0
-                self.shape = to_tensor(-1).to('cpu')
-                self._shape_tuple = (-1,)
+                self.shape = TensorShape([None,shape])
+                self._shape_tuple = (None,shape)
+            elif is_tensor(shape) and 'int' in str(shape.dtype):
+                self.ndim = len(shape)
+
+                shape = to_list(to_numpy(shape))
+                self._shape_tuple = (None,) + tuple(shape)
+                self.shape = TensorShape(self._shape_tuple)
             else:
+                print(shape)
                 self.ndim = len(shape)
                 shape=to_list(to_numpy(shape))
-                self._shape_tuple = (-1,) + tuple(shape)
-                self.shape = to_tensor(self._shape_tuple).int().to('cpu')
+                self._shape_tuple = (None,) + tuple(shape)
+                self.shape = TensorShape(self._shape_tuple)
         else:
             self.ndim = ndim
             self.shape = None
@@ -119,6 +133,10 @@ class TensorSpec(object):
             if max_axis > max_dim:
                 raise ValueError('Axis {} is greater than the maximum allowed value: {}'
                                  .format(max_axis, max_dim))
+    @classmethod
+    def tensor_to_spec(cls, t:Tensor, object_type:ObjectType=None,name=None):
+        return cls(shape=tensor_to_shape(t),dtype=t.dtype,object_type=object_type,name=t.name if name is None else name)
+
 
     @property
     def dtype(self):
@@ -339,8 +357,6 @@ def assert_spec_compatibility(input_spec: TensorSpec, other_spec: TensorSpec):
     """
     if not input_spec:
         return False
-    input_spec.shape.to('cpu')
-    other_spec.shape.to('cpu')
     if isinstance(input_spec, (tuple, list)) and all([isinstance(item, numbers.Integral) for item in input_spec]):
         input_spec = TensorSpec(shape=to_tensor(input_spec))
 
@@ -400,6 +416,9 @@ def assert_spec_compatibility(input_spec: TensorSpec, other_spec: TensorSpec):
     # Check shape.
     if input_spec.shape is not None:
         shape = other_spec._shape_tuple
+        is_compatible=TensorShape(input_spec.shape).is_compatible_with(TensorShape(other_spec._shape_tuple))
+        if is_compatible:
+            return is_compatible
         if shape is not None:
             for spec_dim, dim in zip(other_spec._shape_tuple, input_spec._shape_tuple):
                 if spec_dim is not None and dim is not None:
