@@ -12,7 +12,7 @@ import warnings
 import numpy as np
 
 from trident.backend.common import *
-from trident.backend.load_backend import get_backend
+from trident.backend.common import get_backend
 from trident.callbacks.callback_base import CallbackBase
 from trident.data.image_common import *
 from trident.misc.visualization_utils import *
