@@ -9,20 +9,22 @@ import builtins
 import numpy as np
 import os
 from trident.backend.common import *
-from trident.backend.load_backend import get_backend
+from trident.backend.common import get_backend
 from trident.callbacks.callback_base import CallbackBase
 from trident.data.image_common import *
 from trident.misc.ipython_utils import is_in_colab
 if get_backend()=='pytorch':
     import torch.nn as nn
+    from trident.backend.pytorch_backend import get_device
     from trident.backend.pytorch_ops import to_numpy,to_tensor,arange,shuffle,cast,clip,sqrt,int_shape
     from trident.optims.pytorch_losses import CrossEntropyLoss
 elif get_backend()=='tensorflow':
     import tensorflow as tf
+    from trident.backend.tensorflow_backend import get_device
     from trident.backend.tensorflow_ops import  to_numpy,to_tensor,arange,shuffle,cast,clip,sqrt,int_shape,concate,zeros_like,ones_like
     from trident.optims.tensorflow_losses import CrossEntropyLoss
 
-
+working_directory=get_session().working_directory
 __all__ = ['RegularizationCallbacksBase', 'MixupCallback', 'CutMixCallback']
 
 class RegularizationCallbacksBase(CallbackBase):
@@ -50,17 +52,20 @@ class MixupCallback(RegularizationCallbacksBase):
         self.alpha=alpha
         self.loss_criterion=loss_criterion()
         self.loss_weight=loss_weight
-        self.save_path=save_path
-        if save_path is not None:
-            make_dir_if_need(save_path)
+        if save_path is None:
+            self.save_path = os.path.join(working_directory, 'Results')
+        else:
+            self.save_path = save_path
+        make_dir_if_need(self.save_path)
     def on_loss_calculation_end(self, training_context):
         """Returns mixed inputs, pairs of targets, and lambda"""
+        model = training_context['current_model']
         train_data = training_context['train_data']
         x=None
         y=None
-        x = train_data.value_list[0].copy().detach()  # input
-        y = train_data.value_list[1].copy().detach()  # label
-        model=training_context['current_model']
+        x = train_data.value_list[0].copy().detach().to(model.device)  # input
+        y = train_data.value_list[1].copy().detach().to(model.device)   # label
+
 
         lam=builtins.min(builtins.max(np.random.beta(self.alpha,self.alpha),0.3), 0.7)
 
@@ -71,32 +76,41 @@ class MixupCallback(RegularizationCallbacksBase):
         mixed_x=None
         if get_backend()=='pytorch':
             mixed_x = lam * x + (1 - lam) * x[index, :]
-            pred = model(to_tensor(mixed_x, requires_grad=True))
+            pred = model(to_tensor(mixed_x, requires_grad=True,device=model.device))
             y_a, y_b = y, y[index]
             this_loss = lam * self.loss_criterion(pred, y_a.long()) + (1 - lam) * self.loss_criterion(pred, y_b.long())
+            training_context['current_loss'] = training_context['current_loss'] + this_loss * self.loss_weight
+            if training_context['is_collect_data']:
+                training_context['losses'].collect('mixup_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
+
         elif get_backend()=='tensorflow':
-            x1 = tf.gather(x, index,axis=0)
-            y1 = tf.gather(y, index,axis=0)
-            mixed_x = lam * x + (1 - lam) * x1
-            pred = model(to_tensor(mixed_x, requires_grad=True))
-            y_a, y_b = y, y1
+            with tf.device(get_device()):
+                x1 = tf.gather(x, index,axis=0)
+                y1 = tf.gather(y, index,axis=0)
+                mixed_x = lam * x + (1 - lam) * x1
+                pred = model(to_tensor(mixed_x, requires_grad=True))
+                y_a, y_b = y, y1
 
-            this_loss = lam * self.loss_criterion(pred, y_a) + (1 - lam) * self.loss_criterion(pred,y_b)
+                this_loss = lam * self.loss_criterion(pred, y_a) + (1 - lam) * self.loss_criterion(pred,y_b)
 
-        training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
-        if training_context['is_collect_data']:
-            training_context['losses'].collect('mixup_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
+                training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
+                if training_context['is_collect_data']:
+                    training_context['losses'].collect('mixup_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
 
         if training_context['current_batch']==0:
             for item in mixed_x:
                 if self.save_path is None and not is_in_colab():
                     item = unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = unnormalize(0, 255)(item)
-                    array2image(item).save('Results/mixup_{0}.jpg'.format(get_time_suffix()))
+                    array2image(item).save(os.path.join(self.save_path,'mixup_{0}.jpg'.format(get_time_suffix())))
                 elif self.save_path is not None:
                     item = unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path, 'mixup_{0}.jpg'.format(get_time_suffix())))
+        mixed_x=None
+        x=None
+        y=None
+
 
 
 class CutMixCallback(RegularizationCallbacksBase):
@@ -117,7 +131,11 @@ class CutMixCallback(RegularizationCallbacksBase):
         self.alpha=alpha
         self.loss_criterion=loss_criterion()
         self.loss_weight=loss_weight
-        self.save_path=save_path
+        if save_path is None:
+            self.save_path=os.path.join(working_directory,'Results')
+        else:
+            self.save_path =save_path
+        make_dir_if_need(self.save_path)
 
     def rand_bbox(self, width, height, lam):
         """
@@ -149,13 +167,14 @@ class CutMixCallback(RegularizationCallbacksBase):
 
     def on_loss_calculation_end(self, training_context):
         """Returns mixed inputs, pairs of targets, and lambda"""
+        model = training_context['current_model']
         train_data = training_context['train_data']
         x = None
         y = None
-        x = train_data.value_list[0].copy().detach()   # input
-        y = train_data.value_list[1].copy().detach()   # label
+        x = train_data.value_list[0].copy().detach().to(model.device)    # input
+        y = train_data.value_list[1].copy().detach().to(model.device)    # label
+
 
-        model = training_context['current_model']
 
         lam=builtins.min(builtins.max(np.random.beta(self.alpha,self.alpha),0.1), 0.4)
 
@@ -170,45 +189,50 @@ class CutMixCallback(RegularizationCallbacksBase):
             x[:, :, bbx1:bbx2, bby1:bby2] = x[index, :, bbx1:bbx2, bby1:bby2]
             # adjust lambda to exactly match pixel ratio
             lam = 1 - ((bbx2 - bbx1) * (bby2 - bby1) / (x.shape[3] * x.shape[2]))
-            pred = model(to_tensor(x, requires_grad=True))
+            pred = model(to_tensor(x, requires_grad=True,device=model.device))
             this_loss = lam * self.loss_criterion(pred, y_a.long()) + (1 - lam) * self.loss_criterion(pred, y_b.long())
+            training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
+            if training_context['is_collect_data']:
+                training_context['losses'].collect('cutmix_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
 
         elif get_backend() == 'tensorflow':
-
-            y1 = tf.gather(y,index,axis=0)
-            x1= tf.gather(x,index,axis=0)
-            y_a, y_b = y, y1
-            bbx1, bby1, bbx2, bby2 = self.rand_bbox(x.shape[2], x.shape[1], lam)
-            filter=np.zeros(int_shape(x))
-            filter[:, bbx1:bbx2, bby1:bby2, :] =1
-            filter=to_tensor(x)
-            x=x*(1-filter)+x1*filter
-            #x[:, bbx1:bbx2, bby1:bby2, :] = x1[:, bbx1:bbx2, bby1:bby2,:]
-            # adjust lambda to exactly match pixel ratio
-            lam = 1 - ((bbx2 - bbx1) * (bby2 - bby1) / (x.shape[2] * x.shape[1]))
-            pred = model(to_tensor(x, requires_grad=True))
-            loss1=self.loss_criterion(pred, y_a)
-            loss2=self.loss_criterion(pred, y_b)
-            this_loss = lam *loss1  + (1 - lam) * loss2
-
+            with tf.device(get_device()):
+                y1 = tf.gather(y,index,axis=0)
+                x1= tf.gather(x,index,axis=0)
+                y_a, y_b = y, y1
+                bbx1, bby1, bbx2, bby2 = self.rand_bbox(x.shape[2], x.shape[1], lam)
+                filter=np.zeros(int_shape(x))
+                filter[:, bbx1:bbx2, bby1:bby2, :] =1
+                filter=to_tensor(x)
+                x=x*(1-filter)+x1*filter
+                #x[:, bbx1:bbx2, bby1:bby2, :] = x1[:, bbx1:bbx2, bby1:bby2,:]
+                # adjust lambda to exactly match pixel ratio
+                lam = 1 - ((bbx2 - bbx1) * (bby2 - bby1) / (x.shape[2] * x.shape[1]))
+                pred = model(to_tensor(x, requires_grad=True))
+                loss1=self.loss_criterion(pred, y_a)
+                loss2=self.loss_criterion(pred, y_b)
+                this_loss = lam *loss1  + (1 - lam) * loss2
+                training_context['current_loss'] = training_context['current_loss'] + this_loss * self.loss_weight
+                if training_context['is_collect_data']:
+                    training_context['losses'].collect('cutmix_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
 
         if training_context['current_batch'] == 0:
             if self.save_path is None and not is_in_colab():
                 for item in x:
                     item = unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = unnormalize(0, 255)(item)
-                    array2image(item).save('Results/cutmix_{0}.jpg'.format(get_time_suffix()))
+                    array2image(item).save(os.path.join(self.save_path, 'cutmix_{0}.jpg'.format(get_time_suffix())))
+
             elif self.save_path is not None:
                 for item in x:
                     item = unnormalize([0.485, 0.456, 0.406], [0.229, 0.224, 0.225])(to_numpy(item))
                     item = unnormalize(0, 255)(item)
                     array2image(item).save(os.path.join(self.save_path,'cutmix_{0}.jpg'.format(get_time_suffix())))
+        x=None
+        y=None
 
 
 
-        training_context['current_loss'] = training_context['current_loss'] + this_loss *self.loss_weight
-        if training_context['is_collect_data']:
-            training_context['losses'].collect('cutmix_loss', training_context['steps'], float(to_numpy(this_loss * self.loss_weight)))
 
 
 
