@@ -2,7 +2,6 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import os
-os.environ['TRIDENT_BACKEND'] = 'tensorflow'
 import builtins
 from typing import Optional,List,Tuple
 import collections
@@ -19,12 +18,7 @@ from tensorflow.python.ops import embedding_ops
 from tensorflow.python.eager import context
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import tensor_shape
-from tensorflow.python.keras import initializers
-from tensorflow.python.keras.engine.input_spec import InputSpec
-from tensorflow.python.keras.layers.convolutional import Conv
-from tensorflow.python.keras.utils import conv_utils
-from tensorflow.python.keras.utils.generic_utils import deserialize_keras_object
-from tensorflow.python.keras.utils.generic_utils import serialize_keras_object
+
 from tensorflow.python.ops import gen_math_ops, image_ops, math_ops
 from tensorflow.python.ops import nn, nn_ops, array_ops
 from tensorflow.python.ops import sparse_ops
@@ -32,10 +26,11 @@ from tensorflow.python.ops import standard_ops
 
 from trident.backend.common import *
 from trident.backend.load_backend import *
-from trident.backend.tensorflow_backend import Layer, Sequential,normalize_padding
+from trident.backend.tensorflow_backend import Layer, Sequential, Parameter,normalize_padding, get_device
 from trident.backend.tensorflow_ops import *
 from trident.layers.tensorflow_activations import get_activation
-
+from trident.layers.tensorflow_initializers import *
+from trident.backend import dtype
 _tf_data_format = 'channels_last'
 
 
@@ -59,12 +54,7 @@ __all__ = ['Dense','Embedding' ,'Flatten', 'Concatenate', 'Concate', 'Add', 'Sub
 
 _session = get_session()
 
-_device = 'CPU'
-for device in device_lib.list_local_devices():
-    if tf.DeviceSpec.from_string(device.name).device_type == 'GPU':
-        _device = 'GPU'
-        break
-
+_device =get_device()
 _epsilon = _session.epsilon
 
 
@@ -108,7 +98,8 @@ class Dense(Layer):
             self.num_filters = unpack_singleton(num_filters)
         else:
             raise ValueError('output_shape should be integer, list of integer or tuple of integer...')
-        self.weight = None
+        self.register_parameter('weight', None)
+        self.register_parameter('bias', None)
 
         self.use_bias = use_bias
         if kernel_regularizer == 'l2':
@@ -119,18 +110,26 @@ class Dense(Layer):
         self.activation = get_activation(activation)
 
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
-            if isinstance(input_shape, int):
-                self.input_filters = input_shape
-            self.weight = tf.Variable(tf.random.normal(shape=(self.input_filters,self.num_filters), mean=0, stddev=1) * 0.02, name='weight')
-            if self.use_bias:
-                self.bias = tf.Variable(to_tensor(np.zeros((self.num_filters))), name='bias')
+            with tf.device(get_device()):
+                with self.name_scope:
+                    if len(input_shape.dims) == 1:
+                        self.input_filters = input_shape.dims[0]
+                    else:
+                        self.input_filters = input_shape[self.filter_index]
+                    self.weight =Parameter(data=random_normal(shape=(self.input_filters,self.num_filters), mean=0., std=0.2) , name='weight')
+                    kaiming_uniform(self.weight, a=math.sqrt(5))
+                    if self.use_bias:
+                        self.bias =Parameter(data=random_normal(shape=(self.num_filters), mean=0., std=0.002) , name='bias')
+                    else:
+                        self.bias=None
 
-            self._built = True
+                self._built = True
+
+
+    def forward(self, x, **kwargs) :
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
         if hasattr(self, 'kernel_regularizer') and self.kernel_regularizer is not None:
             x = tf.matmul(x, self.kernel_regularizer(self.weight))
         else:
@@ -163,13 +162,13 @@ class Embedding(Layer):
         self.norm_type = norm_type
         self.scale_grad_by_freq = scale_grad_by_freq
         if _weight is not None and int_shape(_weight)[-1] == embedding_dim and len( int_shape(_weight))==2:
-            self.weight = tf.Variable(_weight)
+            self.weight =Parameter(_weight)
             self.num_embeddings = int_shape(self.weight)[0]
             self._built = True
         elif _weight is not None :
             raise  ValueError('Shape[-1] of weight does not match embedding_dim')
         elif _weight is None and self.num_embeddings is not None:
-            self.weight = tf.Variable(tf.random.normal(shape=(self.num_embeddings, self.embedding_dim), mean=0, stddev=1) * 0.02, name='weight')
+            self.weight =Parameter(tf.random.normal(shape=(self.num_embeddings, self.embedding_dim), mean=0, stddev=1) * 0.02, name='weight')
             self._built = True
         if self._built:
            # self.to(self.device)
@@ -179,24 +178,27 @@ class Embedding(Layer):
         self.sparse = sparse
 
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
-            if isinstance(input_shape, int):
-                self.input_filters = input_shape
-            self.weight = tf.Variable(tf.random.normal(shape=(self.num_embeddings, self.embedding_dim), mean=0, stddev=1) * 0.02, name='weight')
-            if self.use_bias:
-                self.bias = tf.Variable(to_tensor(np.zeros((self.num_filters))), name='bias')
+            with tf.device(get_device()):
+                with self.name_scope:
+                    if len(input_shape.dims) == 1:
+                        self.input_filters = input_shape.dims[0]
+                    else:
+                        self.input_filters = input_shape[self.filter_index]
+                    self.weight =Parameter(tf.random.normal(shape=(self.num_embeddings, self.embedding_dim), mean=0, stddev=1) * 0.02, name='weight')
+                    if self.use_bias:
+                        self.bias =Parameter(to_tensor(np.zeros((self.num_filters))), name='bias')
 
-            self._built = True
+                    self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
 
+    def forward(self, x, **kwargs) :
         dtype = x.dtype
         if dtype != tf.int32 and dtype !=tf.int64:
             x = math_ops.cast(x,tf.int32)
         if isinstance(self.weight, sharded_variable.ShardedVariable):
-            x = embedding_ops.embedding_lookup_v2(self.weight.variables, x)
+            x = embedding_ops.embedding_lookup_v2(self.weight.variables,x )
         else:
             x = embedding_ops.embedding_lookup_v2(self.weight, x)
         return x
@@ -210,12 +212,14 @@ class Flatten(Layer):
         self.keep_output = keep_output
 
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
+
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = tf.reshape(x, [x.get_shape().as_list()[0], -1])
         return x
 
@@ -227,7 +231,8 @@ class Concate(Layer):
         super(Concate, self).__init__()
         self.axis = axis
 
-    def forward(self, *x) -> tf.Tensor:
+
+    def forward(self, x, **kwargs) -> tf.Tensor:
         if not isinstance(x, (list, tuple)) or len(x) < 2:
             raise ValueError('A `Concatenate` layer should be called on a list of at least 2 tensor  inputs')
 
@@ -256,12 +261,13 @@ class Add(Layer):
     def __init__(self):
         super(Add, self).__init__()
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self.output_shape = input_shape
             self._built = True
 
-    def forward(self, *x) -> tf.Tensor:
+
+    def forward(self, x, **kwargs) -> tf.Tensor:
         if not isinstance(x, (list, tuple)):
             raise ValueError('A merge layer should be called on a list of inputs.')
         if isinstance(x, tuple):
@@ -278,12 +284,13 @@ class Subtract(Layer):
     def __init__(self):
         super(Subtract, self).__init__()
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self.output_shape = input_shape
             self._built = True
 
-    def forward(self, *x) -> tf.Tensor:
+
+    def forward(self, x, **kwargs) -> tf.Tensor:
         if not isinstance(x, (list, tuple)):
             raise ValueError('A merge layer should be called on a list of inputs.')
         if isinstance(x, tuple):
@@ -300,12 +307,13 @@ class Dot(Layer):
     def __init__(self, axis=1):
         super(Dot, self).__init__()
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self.output_shape = input_shape
             self._built = True
 
-    def forward(self, *x) -> tf.Tensor:
+
+    def forward(self, x, **kwargs) -> tf.Tensor:
         if not isinstance(x, (list, tuple)):
             raise ValueError('A merge layer should be called on a list of inputs.')
         if isinstance(x, tuple):
@@ -341,8 +349,10 @@ class SoftMax(Layer):
         self.axis = axis
         self.add_noise = add_noise
         self.noise_intensity = noise_intensity
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+
+    def forward(self, x, **kwargs) :
+
         if not hasattr(self, 'add_noise'):
             self.add_noise = False
             self.noise_intensity = 0.005
@@ -385,7 +395,7 @@ class Scale(Layer):
         else :
             raise ValueError('Only [uniform,channel,elementwise] is valid value for mode ')
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         def remove_from(name:str,*dicts):
             for d in dicts:
                 if name in d:
@@ -393,8 +403,8 @@ class Scale(Layer):
         if self._built == False:
             if self.mode =='uniform':
                 self.scale=tf.Variable(self._scale.copy(),trainable=True)
-                self.shift = tf.Variable(self._shift.copy(), trainable=True)
-                self.power = tf.Variable(self._power.copy(), trainable=True)
+                self.shift =Parameter(self._shift.copy(), trainable=True)
+                self.power =Parameter(self._power.copy(), trainable=True)
             elif self.mode =='channel':
                 new_shape = [1] * (len(input_shape) + 1)
                 new_shape[self.filter_index] = self.input_filters
@@ -411,9 +421,9 @@ class Scale(Layer):
                         self._power = repeat_elements(self._power, self.input_filters, 0)
                     self._power = reshape(self._power, new_shape)
 
-                self.scale = tf.Variable(initial_value=self._scale.copy(), trainable=True)
-                self.shift = tf.Variable(initial_value=self._shift.copy(), trainable=True)
-                self.power = tf.Variable(initial_value=self._power.copy(), trainable=True)
+                self.scale =Parameter(data=self._scale.copy(), trainable=True)
+                self.shift =Parameter(data=self._shift.copy(), trainable=True)
+                self.power =Parameter(data=self._power.copy(), trainable=True)
             elif self.mode == 'elementwise':
                 if ndim(self._scale) == 1 and numel(self._scale) ==1:
                     self._scale=ones(input_shape)*self._scale
@@ -428,16 +438,17 @@ class Scale(Layer):
                     self._shift=self._shift.expand_dims(0,1)
                 if int_shape(self._power) ==input_shape:
                     self._power=self._power.expand_dims(0,1)
-                self.scale = tf.Variable(self._scale.copy(), trainable=True)
-                self.shift = tf.Variable(self._shift.copy(), trainable=True)
-                self.power = tf.Variable(self._power.copy(), trainable=True)
+                self.scale =Parameter(self._scale.copy(), trainable=True)
+                self.shift =Parameter(self._shift.copy(), trainable=True)
+                self.power =Parameter(self._power.copy(), trainable=True)
             remove_from('_scale',self.__dict__, self._buffers)
             remove_from('_shift',self.__dict__, self._buffers)
             remove_from('_power',self.__dict__, self._buffers)
             self._built = True
 
-    def forward(self, *x) -> Tensor:
-        x=enforce_singleton(x)
+
+    def forward(self, x, **kwargs) -> Tensor:
+
         x = pow(x*self.scale+self.shift,self.power)
         return x
 
@@ -545,6 +556,8 @@ class _ConvNd(Layer):
         self.separable = separable
         if self.separable == True:
             self.depthwise = True
+        self.register_parameter('weight', None)
+        self.register_parameter('bias', None)
 
 
 
@@ -552,63 +565,83 @@ class _ConvNd(Layer):
         self.transposed = transposed
         self.use_bias = use_bias
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
-            with self.name_scope:
-                self.input_filters = to_numpy(input_shape)[self.filter_index]
-                if self.depthwise or self.separable:
-                    if self.depth_multiplier is None:
-                        self.depth_multiplier = 1
-                        # ex. self.depth_multiplier=0.5  input 128==>output 64  groups=64
-                        # ex. self.depth_multiplier=2  input 64==>output 128  groups=64
-                    self.groups = int(builtins.round(self.input_filters * builtins.min(builtins.max(self.depth_multiplier, 0), 1), 0))
+            with tf.device(get_device()):
+                with self.name_scope:
+                    self.input_filters =input_shape[self.filter_index]
+
+                    if self.auto_pad:
+                        if self.transposed == False:
+                            padding = get_static_padding(self.rank, self.kernel_size, self.strides, self.dilation, input_shape[1:-1])
+                            self.padding = tuple(padding)
+                        else:
+                            self.padding, self.output_padding = get_static_padding(self.rank, self.kernel_size, self.strides, self.dilation, input_shape[1:-1],self.transposed)
+                    else:
+                        if self.padding is None:
+                            self.padding = [0] * (2 * self.rank)
+                        elif isinstance(self.padding, int):
+                            self.self.padding = [self.padding] * (2 * self.rank)
+                        elif len(self.padding) == self.rank:
+                            self.padding = list(self.padding) * 2
+                        elif len(self.padding) == 2 * self.rank:
+                            pass
+
+                    if self.depthwise or self.separable:
+                        if self.depth_multiplier is None:
+                            self.depth_multiplier = 1
+                            # ex. self.depth_multiplier=0.5  input 128==>output 64  groups=64
+                            # ex. self.depth_multiplier=2  input 64==>output 128  groups=64
+                        self.groups = int(builtins.round(self.input_filters * builtins.min(builtins.max(self.depth_multiplier, 0), 1), 0))
 
 
-                if self.num_filters is None and self.depth_multiplier is not None:
-                    self.num_filters=int(builtins.round(self.input_filters* self.depth_multiplier))
+                    if self.num_filters is None and self.depth_multiplier is not None:
+                        self.num_filters=int(builtins.round(self.input_filters* self.depth_multiplier))
 
-                if self.groups != 1 and self.num_filters % self.groups != 0:
-                    raise ValueError('out_channels must be divisible by groups')
+                    if self.groups != 1 and self.num_filters % self.groups != 0:
+                        raise ValueError('out_channels must be divisible by groups')
 
-                if self.depthwise and self.num_filters % self.groups != 0:
-                    raise ValueError('out_channels must be divisible by groups')
+                    if self.depthwise and self.num_filters % self.groups != 0:
+                        raise ValueError('out_channels must be divisible by groups')
 
-                if self.auto_pad:
-                    self.padding = get_static_padding(self.rank, self.kernel_size ,
-                                                       self.strides ,  self.dilation ,
-                                                       tuple(to_list(input_shape)[:-1]))
-                else:
-                    self.padding = normalize_padding(self.padding, self.rank)
-
-
-
-                    # elif self.depth_multiplier < 1:
-                    #
-                    #     self.groups = int(builtins.round(self.input_filters * self.depth_multiplier, 0))
-                    #     self.num_filters=int(self.groups)
+                    # if self.auto_pad:
+                    #     self.padding = get_static_padding(self.rank, self.kernel_size ,
+                    #                                        self.strides ,  self.dilation ,
+                    #                                        tuple(to_list(input_shape)[:-1]))
                     # else:
+                    #     self.padding = normalize_padding(self.padding, self.rank)
+
+
+
+                        # elif self.depth_multiplier < 1:
+                        #
+                        #     self.groups = int(builtins.round(self.input_filters * self.depth_multiplier, 0))
+                        #     self.num_filters=int(self.groups)
+                        # else:
 
 
 
 
-                channel_multiplier = int(self.num_filters // self.groups) if self.groups>1 else int(self.num_filters)#if self.depth_multiplier is None else self.depth_multiplier  # default channel_multiplier
+                    channel_multiplier = int(self.num_filters // self.groups) if self.groups>1 else int(self.num_filters)#if self.depth_multiplier is None else self.depth_multiplier  # default channel_multiplier
 
-                if self.transposed:
-                    # filter_height, filter_width,  out_channels in_channels,
-                    self.weight = tf.Variable(tf.random.normal(shape=[*self.kernel_size, int(channel_multiplier), int(self.input_filters)], mean=0,  stddev=1) * 0.02, trainable=True, name='weight')
-                else:
+                    if self.transposed:
+                        # filter_height, filter_width,  out_channels in_channels,
+                        self.weight =Parameter(tf.random.normal(shape=[*self.kernel_size, int(channel_multiplier), self.input_filters], mean=0,  stddev=1) * 0.02, trainable=True, name='weight')
 
-                    # [filter_height, filter_width, in_channels, out_channels]`
-                    self.weight = tf.Variable(  tf.random.normal(shape=[*self.kernel_size, int(self.input_filters), int(channel_multiplier)], mean=0,    stddev=1) * 0.02, trainable=True, name='weight')
+                    else:
 
-                    if self.separable:
-                        pointwise_kernel_size = (1,) * len(self.kernel_size)
-                        self.pointwise = tf.Variable(tf.random.normal( shape=[*pointwise_kernel_size, int(self.input_filters * channel_multiplier),int(self.num_filters)], mean=0, stddev=1) * 0.02, trainable=True, name='weight')
+                        # [filter_height, filter_width, in_channels, out_channels]`
+                        self.weight =Parameter( data=tf.random.normal(shape=[*self.kernel_size, self.input_filters, int(channel_multiplier)], mean=0,    stddev=1) * 0.02, trainable=True, name='weight')
 
-                if self.use_bias:
-                    self.bias = tf.Variable(tf.random.normal([int(self.num_filters)]), name='bias')
+                        if self.separable:
+                            pointwise_kernel_size = (1,) * len(self.kernel_size)
+                            self.pointwise =Parameter(data=tf.random.normal( shape=[*pointwise_kernel_size, self.input_filters * channel_multiplier,self.num_filters], mean=0, stddev=1) * 0.02, trainable=True, name='weight')
 
-            self._built = True
+                    if self.use_bias:
+                        self.bias =Parameter(data=tf.random.normal([int(self.num_filters)]), name='bias')
+
+
+                self._built = True
 
     def extra_repr(self):
         s = 'kernel_size={kernel_size}, {num_filters},strides={strides}'
@@ -663,12 +696,14 @@ class Conv1d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=1
 
-    def conv1d_forward(self, x):
+
+    def conv1d_forward(self, x, **kwargs):
         return tf.nn.conv1d(x, filters=self.weight, stride=(1,) + self.strides + (1,), padding='SAME' if self.auto_pad else 'VALID',
                             data_format="NWC", dilations=(1,) + self.dilation + (1,), name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv1d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -820,10 +855,12 @@ class Conv2d(_ConvNd):
         self.rank = 2
 
 
-    def conv2d_forward(self, x):
+    def conv2d_forward(self, x, **kwargs):
         if self.auto_pad == True and len(self.padding) == self.rank + 2:
             x = tf.pad(x, self.padding, mode='CONSTANT')
         else:
+            if len(self.padding)==4:
+                self.padding=((self.padding[0],self.padding[1]),(self.padding[2],self.padding[3]))
             padlist = list(self.padding)
             padlist.insert(0, (0, 0))
             padlist.append((0, 0))
@@ -834,8 +871,8 @@ class Conv2d(_ConvNd):
                             data_format="NHWC", dilations=(1,) + self.dilation + (1,), name=self._name)
 
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+    def forward(self, x, **kwargs) :
+
         x = self.conv2d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -876,12 +913,13 @@ class Conv3d(_ConvNd):
         self.rank = 3
 
 
-    def conv3d_forward(self, x):
+    def conv3d_forward(self, x, **kwargs):
         return tf.nn.conv3d(x, filters=self.weight, strides=(1,) + self.strides + (1,), padding='VALID',
                             data_format="NHWC", dilations=(1,) + self.dilation + (1,), name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv3d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -922,8 +960,8 @@ class TransConv1d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank = 1
 
-    @tf.function
-    def conv1d_forward(self, x):
+
+    def conv1d_forward(self, x, **kwargs):
         in_shape = x.get_shape().as_list()
         in_shape[1] *= self.strides[0]
         in_shape[-1] = self.num_filters
@@ -932,8 +970,9 @@ class TransConv1d(_ConvNd):
                                       padding='SAME' if self.auto_pad else 'VALID', data_format="NHWC", dilations=(1,) + self.dilation + (1,),
                                       name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv1d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -990,7 +1029,8 @@ class TransConv2d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=2
 
-    def conv2d_forward(self, x):
+
+    def conv2d_forward(self, x, **kwargs):
         in_shape = to_list(int_shape(x))
         in_shape[1] *= self.strides[0]
         in_shape[2] *= self.strides[1]
@@ -999,8 +1039,9 @@ class TransConv2d(_ConvNd):
                                       padding='SAME' if self.auto_pad else 'VALID', data_format="NHWC", dilations=(1,) + self.dilation + (1,),
                                       name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv2d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -1056,7 +1097,8 @@ class TransConv3d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=3
 
-    def conv3d_forward(self, x):
+
+    def conv3d_forward(self, x, **kwargs):
         in_shape = x.get_shape().as_list()
         in_shape[1] *= self.strides[0]
         in_shape[2] *= self.strides[1]
@@ -1066,8 +1108,9 @@ class TransConv3d(_ConvNd):
                                       padding='SAME' if self.auto_pad else 'VALID', data_format="NHWC", dilations=(1,) + self.dilation + (1,),
                                       name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv3d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -1113,12 +1156,14 @@ class DepthwiseConv1d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=1
 
-    def conv1d_forward(self, x):
+
+    def conv1d_forward(self, x, **kwargs):
         return tf.nn.convolution(x, filters=self.weight, strides=(1,) + self.strides + (1,), padding=self.padding,
                                  data_format="NHWC", dilations=(1,) + self.dilation + (1,), name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv1d_forward(x)
         if self.use_bias:
             x += self.bias
@@ -1156,7 +1201,8 @@ class DepthwiseConv2d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=2
 
-    def conv2d_forward(self, x):
+
+    def conv2d_forward(self, x, **kwargs):
         if self.auto_pad == True and len(self.padding) == self.rank + 2:
             x = tf.pad(x, self.padding, mode='CONSTANT')
         else:
@@ -1168,8 +1214,9 @@ class DepthwiseConv2d(_ConvNd):
 
         return tf.nn.depthwise_conv2d(x,filter=self.weight, strides=(1,) + self.strides + (1,), padding='VALID', data_format="NHWC", dilations= self.dilation, name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv2d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -1200,12 +1247,14 @@ class DepthwiseConv3d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=3
 
-    def conv3d_forward(self, x):
+
+    def conv3d_forward(self, x, **kwargs):
         return tf.nn.convolution(x, filters=self.weight, strides=(1,) + self.strides + (1,), padding='VALID',
                                  data_format="NHWC", dilations=(1,) + self.dilation + (1,), name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv3d_forward(x)
         if self.use_bias:
             x += self.bias
@@ -1244,7 +1293,8 @@ class SeparableConv2d(_ConvNd):
         self.activation = get_activation(activation)
         self.rank=2
 
-    def conv2d_forward(self, x):
+
+    def conv2d_forward(self, x, **kwargs):
         if self.auto_pad == True and len(self.padding) == self.rank + 2:
             x = tf.pad(x, self.padding, mode='CONSTANT')
         else:
@@ -1258,8 +1308,9 @@ class SeparableConv2d(_ConvNd):
                                       strides=(1,) + self.strides + (1,), padding='VALID', data_format="NHWC",
                                       dilations=(1,) + self.dilation + (1,), name=self._name)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = self.conv2d_forward(x)
         if self.use_bias:
             x = tf.nn.bias_add(x, self.bias, data_format='NHWC')
@@ -1290,7 +1341,7 @@ class GatedConv2d(Conv2d):
         self.norm = norm
 
 
-    def forward(self, x):
+    def forward(self, x, **kwargs):
         x = super().forward(x)
         if self.strides==(2,2):
             x = x[:, 1::2, 1::2,:]
@@ -1324,8 +1375,9 @@ class Upsampling2d(Layer):
         self.align_corners = align_corners
         self.keep_output = keep_output
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         new_shape =list(int_shape(x)[1:])
 
         if self.scale_factor is not None and isinstance(self.scale_factor, tuple):
@@ -1363,11 +1415,12 @@ class Lambda(Layer):
         super(Lambda, self).__init__(name=name)
         self.function = function
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self._built = True
 
-    def forward(self, *x):
+
+    def forward(self, x, **kwargs) :
         return self.function(*x)
 
     def extra_repr(self):
@@ -1379,12 +1432,13 @@ class Reshape(Layer):
         super(Reshape, self).__init__(name=name)
         self.target_shape = target_shape
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         x = tf.reshape(x, tf.constant((x.get_shape()[0], *self.target_shape), dtype=tf.int32))
         return x
 
@@ -1399,12 +1453,13 @@ class Dropout(Layer):
         self.dropout_rate = dropout_rate
         self.keep_output = keep_output
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         if self.training:
             x = tf.nn.dropout(x, self.dropout_rate)
         return x
@@ -1417,12 +1472,13 @@ class Noise(Layer):
     def __init__(self, stddev=0.1, name=None):
         super(Noise, self).__init__(stddev=stddev, name=name)
         self.stddev=stddev
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         if self.training:
             noise = random_normal_like(x,mean=mean(x),std=self.stddev)
             x=x+noise
@@ -1439,13 +1495,15 @@ class SingleImageLayer(Layer):
             self.origin_image = to_tensor(image)
             self.input_shape = image.shape[1:]
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
-            self.weight = tf.Variable(to_tensor(self.origin_image.clone()), name='weight')
-            self.input_filters = to_numpy(input_shape)[self.filter_index]
-            self._built = True
+            with tf.device(get_device()):
+                self.weight =Parameter(to_tensor(self.origin_image.clone()), name='weight')
+                self.input_filters =input_shape[self.filter_index]
+                self._built = True
 
-    def forward(self, x):
+    @tf.function
+    def forward(self, x, **kwargs):
         return expand_dims(self.weight, 0)
 
     def extra_repr(self):
