@@ -2,25 +2,22 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 import inspect
+import math
 import numbers
 import copy
 import tensorflow as tf
 import numpy as np
 from tensorflow.python.ops import variables as tf_variables
 from trident.backend.common import get_session, addindent, enforce_singleton, unpack_singleton, get_time_suffix, get_class, \
-    format_time, get_terminal_size, snake2camel, camel2snake,Signature
+    format_time, get_terminal_size, snake2camel, camel2snake,Signature,epsilon,TensorShape
 from trident.backend.tensorflow_backend import Layer, Sequential
 from trident.backend.tensorflow_ops import *
-from  trident.layers import initializers
+from trident.layers.tensorflow_initializers import *
 
-__all__ = ['InstanceNorm','InstanceNorm2d','InstanceNorm3d','BatchNorm','BatchNorm2d','BatchNorm3d','GroupNorm','GroupNorm2d','GroupNorm3d','LayerNorm','LayerNorm2d','LayerNorm3d','PixelNorm','EvoNormB0','EvoNormS0','get_normalization']
+
+__all__ = ['InstanceNorm','InstanceNorm2d','InstanceNorm3d','BatchNorm','BatchNorm2d','BatchNorm3d','GroupNorm','GroupNorm2d','GroupNorm3d','LayerNorm','LayerNorm2d','LayerNorm3d','L2Norm','PixelNorm','EvoNormB0','EvoNormS0','get_normalization']
 
 _session = get_session()
-
-
-
-
-
 _epsilon = _session.epsilon
 
 
@@ -87,7 +84,7 @@ class BatchNorm(Layer):
         https://arxiv.org/abs/1502.03167
 
     """
-    def __init__(self, momentum=0.1, affine=True, track_running_stats=True, axis=-1,renorm=False,eps=1e-5,name=None, **kwargs):
+    def __init__(self, momentum=0.1, affine=True, track_running_stats=True, axis=-1,renorm=False,eps=1e-8,name=None, **kwargs):
         """
          Args:
          eps: a value added to the denominator for numerical stability.
@@ -122,70 +119,87 @@ class BatchNorm(Layer):
         self.momentum = momentum
         self.affine = affine
         self.track_running_stats = track_running_stats
-        self.weight=None
-        self.bias=None
-        self.weight=None # gamma//scale
-        self.bias=None # beta/ offset
+
+        self.register_parameter("weight",None) # gamma//scale
+        self.register_parameter("bias",None) # beta/ offset
 
         self.renorm=renorm
 
 
     def reset_running_stats(self):
         if self.track_running_stats:
-            self.running_mean.assign(tf.zeros(shape=[self.input_filters]))
-            self.running_var.assign(tf.ones(shape=[self.input_filters]))
-            self.num_batches_tracked.assign(0)
+            self.register_buffer("running_mean", zeros(shape=[self.input_filters]))
+            self.register_buffer("running_var", ones(shape=[self.input_filters]))
+            self.register_buffer("num_batches_tracked", to_tensor(0, dtype=tf.int64), persistent=False)
+
+
         if self.affine :
-            self.weight = tf.Variable(tf.ones(shape=[self.input_filters]), trainable=True, name='weight')  # gamma//scale
-            self.bias = tf.Variable(tf.zeros(shape=[self.input_filters]), trainable=True, name='bias')  # beta/ offset
+            self.register_parameter("weight",tf.Variable(tf.ones(shape=[self.input_filters]), trainable=True, name='weight'))  # gamma//scale
+            self.register_parameter("bias",tf.Variable(tf.zeros(shape=[self.input_filters]), trainable=True, name='bias'))  # beta/ offset
 
 
     def assign_moving_average(self, variable, value, momentum, inputs_size):
-            with tf.name_scope('AssignMovingAvg') as scope:
-                decay = to_tensor(1.0 - momentum)
-                if decay.dtype != variable.dtype.base_dtype:
-                    decay = tf.cast(decay, variable.dtype.base_dtype)
-                update_delta = (variable - tf.cast(value, variable.dtype)) * decay
-                if inputs_size is not None:
-                    update_delta = tf.where(inputs_size > 0, update_delta, tf.zeros_like(update_delta))
-                return variable.assign_sub(update_delta, name=scope)
+        with tf.name_scope('AssignMovingAvg') as scope:
+            decay = to_tensor(1.0 - momentum,device=self.get_root().device,dtype=variable.dtype.base_dtype)
+            update_delta = (variable - tf.cast(value, variable.dtype)) * decay
+            if inputs_size is not None:
+                update_delta = where(inputs_size > 0, update_delta, zeros_like(update_delta))
+            variable = tf.math.subtract(variable, update_delta, name=scope)
+            return variable
 
-
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False:
-            self.input_filters= to_numpy(input_shape)[self.filter_index]
-
+            self.input_filters= input_shape[self.filter_index]
             ndims = len(input_shape)
-
             # Convert axis to list and resolve negatives
+            if isinstance(self.axis, int):
+                self.axis = [self.axis]
+            elif isinstance(self.axis, tuple):
+                self.axis = list(self.axis)
+            for idx, x in enumerate(self.axis):
+                if x < 0:
+                    self.axis[idx] = ndims + x
+
+                # Validate axes
+            for x in self.axis:
+                if x < 0 or x >= ndims:
+                    raise ValueError('Invalid axis: %d' % x)
+            if len(self.axis) != len(set(self.axis)):
+                raise ValueError('Duplicate axis: {}'.format(tuple(self.axis)))
+            param_shape = [input_shape[dim] for dim in self.axis]
 
             if self.affine:
-                self.weight=tf.Variable(tf.ones(shape=[self.input_filters]),trainable=True, name='weight') #gamma//scale
-                self.bias=tf.Variable(tf.zeros(shape=[self.input_filters]),trainable=True, name='bias') #beta/ offset
+                self.weight=tf.Variable(tf.ones(shape=param_shape),trainable=True, name='weight') #gamma//scale
+                self.bias=tf.Variable(tf.zeros(shape=param_shape),trainable=True, name='bias') #beta/ offset
 
             if self.track_running_stats:
-                self.register_buffer('running_mean',tf.Variable(tf.zeros(shape=[self.input_filters]), trainable=False,name='running_mean',synchronization=tf.VariableSynchronization.AUTO,aggregation=tf_variables.VariableAggregation.MEAN))
-                self.register_buffer('running_var',tf.Variable(tf.ones(shape=[self.input_filters]), trainable=False, name='running_var',synchronization=tf.VariableSynchronization.AUTO,aggregation=tf_variables.VariableAggregation.MEAN))
-                self.register_buffer('num_batches_tracked',to_tensor(0,dtype=tf.int64))
-
+                self.register_buffer('running_mean', zeros(shape=param_shape))
+                self.register_buffer('running_var', ones(shape=param_shape))
+                self.register_buffer('num_batches_tracked', to_tensor(0, dtype=tf.int64), persistent=False)
 
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+    def forward(self, x, **kwargs) :
+
         input_shape = x.shape
-        ndims= len(x.shape)
-        reduction_axes = [i for i in range(len(x.shape)) if i not in self.axis]
+        ndims = len(input_shape)
+        reduction_axes = [i for i in range(ndims) if i not in self.axis]
+        # Broadcasting only necessary for norm when the axis is not just
+        # the last dimension
         broadcast_shape = [1] * ndims
-        broadcast_shape[self.axis[0]] = input_shape.dims[self.axis[0]].value
+        for dim in self.axis:
+            broadcast_shape[dim] = input_shape.dims[dim].value
+
         def _broadcast(v):
-            if v is not None and len(v.shape) != ndims and reduction_axes != list(range(ndims - 1)):
-                return tf.reshape(v, broadcast_shape)
+            if (v is not None and len(v.shape) != ndims and self.axis != [ndims - 1]):
+                return reshape(v, broadcast_shape)
             return v
 
         scale, offset = _broadcast(self.weight), _broadcast(self.bias)
 
-        mean, variance = tf.nn.moments(x, axes=reduction_axes, keepdims=True)
+        mean, variance = moments(x, axis=reduction_axes, keepdims=True)
+
         running_mean = self.running_mean
         running_var = self.running_var
 
@@ -195,7 +209,7 @@ class BatchNorm(Layer):
         new_mean, new_variance = mean, variance
         def _do_update(var, value):
             """Compute the updates for mean and variance."""
-            return self.assign_moving_average(var, value, self.momentum, self.input_shape.prod())
+            return self.assign_moving_average(var, value, self.momentum, self.input_shape[0])
 
         def mean_update():
             """Update the moving variance."""
@@ -209,18 +223,7 @@ class BatchNorm(Layer):
         def variance_update():
             """Update the moving variance."""
 
-            def true_branch_renorm():
-                # We apply epsilon as part of the moving_stddev to mirror the training
-                # code path.
-                running_stddev = _do_update(sqrt(self.running_var), sqrt(new_variance + self.epsilon))
-                self.running_var.assign(tf.nn.relu(running_stddev * running_stddev - self.epsilon), name='AssignNewValue')
-                return self.running_var
-
-            if self.renorm:
-                true_branch = true_branch_renorm
-            else:
-                true_branch = lambda: _do_update(self.running_var, new_variance)
-
+            true_branch = lambda: _do_update(self.running_var, new_variance)
             false_branch = lambda: self.running_var
             if self.training:
                 return true_branch
@@ -231,6 +234,9 @@ class BatchNorm(Layer):
         variance_update()
 
         return tf.nn.batch_normalization(x, self.running_mean, self.running_var, offset,scale, self.eps)
+
+
+
     def extra_repr(self):
         return '{input_filters}, eps={eps}, momentum={momentum}, affine={affine}, ' \
                'track_running_stats={track_running_stats}'.format(**self.__dict__)
@@ -288,7 +294,7 @@ class GroupNorm(Layer):
         self.axis=axis
 
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False :
             assert  self.input_filters % self.num_groups == 0, 'number of groups {} must divide number of channels {}'.format(self.num_groups,  self.input_filters)
             if self.affine:
@@ -300,8 +306,10 @@ class GroupNorm(Layer):
                 self.register_parameter('bias', None)
 
                 self._built = True
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+
+    def forward(self, x, **kwargs) :
+
         # Prepare broadcasting shape.
         origin_shape=list(int_shape(x))
         group_shape =list(int_shape(x))
@@ -399,7 +407,8 @@ class InstanceNorm(GroupNorm):
         """
         super().__init__(self, num_groups=1,affine=affine,axis=axis, eps=eps, **kwargs)
 
-    def build(self, input_shape):
+
+    def build(self, input_shape:TensorShape):
         if self._built == False:
             self.num_groups=self.input_filters
             if self.affine:
@@ -450,7 +459,7 @@ class LayerNorm(Layer):
     .. _`Layer Normalization`: https://arxiv.org/abs/1607.06450
 
     """
-    def __init__(self, normalized_shape, eps=1e-5, elementwise_affine=True,name=None, **kwargs):
+    def __init__(self,  eps=1e-5, affine=True,name=None, **kwargs):
         """
     Args:
         normalized_shape (int or list or torch.Size): input shape from an expected input
@@ -483,21 +492,21 @@ class LayerNorm(Layer):
 
         """
         super().__init__(name=name)
-        if isinstance(normalized_shape, numbers.Integral):
-            normalized_shape = (normalized_shape,)
-        self.normalized_shape = tuple(normalized_shape)
+
         self.eps = eps
-        self.elementwise_affine = elementwise_affine
+        self.affine = affine
 
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False :
-            if self.elementwise_affine:
-                self.weight = tf.Variable(tf.ones(shape=self.normalized_shape), name='weight') #gamma//scale
-                self.bias = tf.Variable(tf.zeros(shape=self.normalized_shape), name='bias') #beta/ offset
+            if self.affine:
+                self.weight = tf.Variable(tf.ones(shape=self.input_filters), name='weight') #gamma//scale
+                self.bias = tf.Variable(tf.zeros(shape=self.input_filters), name='bias') #beta/ offset
             self._built=True
-    def forward(self, *x):
-        x = enforce_singleton(x)
+
+
+    def forward(self, x, **kwargs) :
+
         mean = x.mean(dim=self.axis, keepdim=True).detach()
         std = x.std(dim=self.axis, keepdim=True).detach()
         return self.weight * (x - mean) / (std + self._eps) +self.bias
@@ -507,13 +516,31 @@ LayerNorm2d=LayerNorm
 LayerNorm3d=LayerNorm
 
 
+
+class L2Norm(Layer):
+    def __init__(self,in_sequence=False, axis=-1,name=None, **kwargs):
+        super().__init__(in_sequence=in_sequence,name=name)
+        self.eps=epsilon()
+        self.axis=axis
+
+    def build(self, input_shape:TensorShape):
+        if self._built == False :
+            self._built = True
+    def forward(self, x, **kwargs):
+        x= l2_normalize(x,axis=self.axis,keepdims=True)
+        return x
+
+
+
+
 class PixelNorm(Layer):
     def __init__(self,eps=1e-5, axis=-1, name=None,**kwargs):
         super(PixelNorm, self).__init__(name=name)
         self.eps=eps
         self.axis=axis
 
-    def forward(self, x):
+
+    def forward(self, x, **kwargs):
         return x /sqrt(mean(x ** 2, axis=self.axis, keepdims=True) + self.eps)
 
 
@@ -523,10 +550,13 @@ class SpectralNorm(Layer):
        layer: tensorflow keras layers (with kernel attribute)
     """
 
-    def __init__(self, layer, **kwargs):
-        super(SpectralNorm, self).__init__(layer, **kwargs)
+    def __init__(self, module, name='weight', power_iterations=1, in_sequence=False, **kwargs):
+        super(SpectralNorm, self).__init__(in_sequence=in_sequence, name=name)
+        self.module = module
+        self.name = name
+        self.power_iterations = power_iterations
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         """Build `Layer`"""
 
         if not self.layer.built:
@@ -538,12 +568,18 @@ class SpectralNorm(Layer):
 
             self.w = self.layer.kernel
             self.w_shape = self.w.shape.as_list()
-            self.u = self.add_variable(shape=tuple([1, self.w_shape[-1]]),
-                initializer=initializers.TruncatedNormal(stddev=0.02), name='sn_u', trainable=False,
-                dtype=tf.float32)
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+            self.u = tf.Variable(zeros(tuple([1, self.w_shape[2]])), trainable=False)
+            self.v =  tf.Variable(zeros(tuple([1, self.w_shape[1]])), trainable=False)
+            kaiming_normal( self.u,a=math.sqrt(5))
+            kaiming_normal(self.v, a=math.sqrt(5))
+
+            self.u.assign(l2_normalize( self.u .value().detach()))
+            self.v.assign(l2_normalize( self.v .value().detach()))
+            self.w_bar = tf.Variable(self.w .value().detach(), requires_grad=False)
+
+    def forward(self, x, **kwargs) :
+
         if self.training == True:
             # Recompute weights for each forward pass
             self._compute_weights()
@@ -551,6 +587,7 @@ class SpectralNorm(Layer):
         output = self.layer(x)
         return output
 
+
     def _compute_weights(self):
         """Generate normalized weights.
         This method will update the value of self.layer.kernel with the
@@ -577,7 +614,8 @@ class EvoNormB0(Layer):
         self.momentum = momentum
         self.eps = eps
 
-    def build(self, input_shape):
+
+    def build(self, input_shape:TensorShape):
         if self._built == False :
             newshape=np.ones(self.rank+2)
             newshape[-1]=self.input_filters
@@ -589,7 +627,7 @@ class EvoNormB0(Layer):
             self.register_buffer('running_var', ones(newshape))
             self._built=True
 
-    def forward(self, x):
+    def forward(self, x, **kwargs):
         if self.training:
             permute_pattern=np.arange(0,self.rank+2)
             permute_pattern[0]=1
@@ -613,7 +651,8 @@ class EvoNormS0(Layer):
         self.nonlinear = nonlinear
         self.groups = groups
 
-    def build(self, input_shape):
+
+    def build(self, input_shape:TensorShape):
         if self._built == False :
             newshape = np.ones(self.rank + 2)
             newshape[-1] = self.input_filters
@@ -625,7 +664,7 @@ class EvoNormS0(Layer):
             self.register_buffer('running_var', ones(newshape))
             self._built=True
 
-    def forward(self, x):
+    def forward(self, x, **kwargs):
         if self.nonlinear:
             num = sigmoid(self.v * x)
             std = group_std(x,self.groups)
