@@ -20,7 +20,7 @@ from torch._six import container_abcs
 from torch.nn import init
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential,  summary
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential,  summary,get_device
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
 from trident.layers.pytorch_activations import get_activation, Identity, Relu
@@ -33,7 +33,7 @@ from trident.optims.pytorch_trainer import *
 __all__ = ['DeeplabV3_plus','DeeplabV3']
 
 _session = get_session()
-_device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+_device = get_device()
 _epsilon=_session.epsilon
 _trident_dir=_session.trident_dir
 _backend = _session.backend
@@ -79,14 +79,13 @@ class ASPP(Layer):
 
         self.project =Conv2d_Block( (1,1),num_filters,strides=1,use_bias=False, bias=False,activation='relu',normalization='batch',dilation=1,dropout_rate=0.5)
 
-    def build(self, input_shape):
+    def build(self, input_shape:TensorShape):
         if self._built == False :
             self.convs.add_module('aspp_pooling',ASPPPooling(self.num_filters, to_list(input_shape[-2:])))
             self.to(self.device)
             self._built = True
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+    def forward(self, x,**kwargs):
         x=self.convs(x)
         x=self.project(x)
         return x
@@ -124,8 +123,8 @@ class _DeeplabV3_plus(Layer):
         self.num_filters=num_filters
         self.classes=classes
         self.atrous_rates=atrous_rates
-        self.backbond1=Sequential(backbond[:low_level_idx])
-        self.backbond2 = Sequential(backbond[low_level_idx:high_level_idx])
+        self.backbond1=Sequential(*backbond[:low_level_idx])
+        self.backbond2 = Sequential(*backbond[low_level_idx:high_level_idx])
         self.aspp=ASPP(atrous_rates=self.atrous_rates,num_filters=self.num_filters)
         self.low_level_conv=Conv2d_Block((1,1),num_filters=int(48*self.num_filters/256),strides=1,use_bias=False,activation='leaky_relu',normalization='batch')
         self.decoder=Sequential(
@@ -136,8 +135,7 @@ class _DeeplabV3_plus(Layer):
 
         )
 
-    def forward(self, *x):
-        x = enforce_singleton(x)
+    def forward(self, x,**kwargs):
         low_level_feature=self.backbond1(x)
         high_level_feature = self.backbond2(low_level_feature)
         x=self.aspp(high_level_feature)
