@@ -20,6 +20,7 @@ from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
 
+from trident.models.pretrained_utils import _make_recovery_model_include_top
 from trident.backend.common import *
 from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, fix_layer, get_device, load
 from trident.data.image_common import *
@@ -235,28 +236,13 @@ def EfficientNetB0(include_top=True, pretrained=True,freeze_features=False, inpu
     if pretrained == True:
         download_model_from_google_drive('1bxnoDerzoNfiZZLft4ocD3DAgx4v6aTN', dirname, 'efficientnet-b0.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b0.pth')))
-        recovery_model.input_shape = input_shape
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb0.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb0.class_names = []
-
-        recovery_model.to(_device)
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb0.model = recovery_model
+    else:
+        effb0.model = _make_recovery_model_include_top( effb0.model, include_top=include_top, classes=classes, freeze_features=False)
 
+    effb0.model .input_shape = input_shape
+    effb0.model .to(_device)
     return effb0
 
 
@@ -269,26 +255,13 @@ def EfficientNetB1(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1F3BtnAjmDz4G9RS9Q0hqU_K7WWXCni1G', dirname, 'efficientnet-b1.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b1.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.eval()
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb1.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb1.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb1.model = recovery_model
+    else:
+        effb1.model = _make_recovery_model_include_top( effb1.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb1.model .input_shape = input_shape
+    effb1.model .to(_device)
     return effb1
 
 
@@ -301,25 +274,13 @@ def EfficientNetB2(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1PjqhB7WJasF_hqOwYtSBNSXSGBY-cRLU', dirname, 'efficientnet-b2.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b2.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb2.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb2.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb2.model = recovery_model
+    else:
+        effb2.model = _make_recovery_model_include_top( effb2.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb2.model .input_shape = input_shape
+    effb2.model .to(_device)
     return effb2
 
 
@@ -332,25 +293,13 @@ def EfficientNetB3(include_top=True, pretrained=True,freeze_features=False, inpu
     if pretrained == True:
         download_model_from_google_drive('11tMxdYdFfaEREwnESO4cwjtcoEB42zB_', dirname, 'efficientnet-b3.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b3.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb3.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb3.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb3.model = recovery_model
+    else:
+        effb3.model = _make_recovery_model_include_top(effb3.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb3.model.input_shape = input_shape
+    effb3.model.to(_device)
     return effb3
 
 
@@ -363,26 +312,13 @@ def EfficientNetB4(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1X4ZOBR_ETRHZJeffJHvCmWTTy9_aW8SP', dirname, 'efficientnet-b4.pth')
         recovery_model =fix_layer( load(sanitize_path(os.path.join(dirname, 'efficientnet-b4.pth'))))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb4.class_names = []
-
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb4.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb4.model = recovery_model
+    else:
+        effb4.model = _make_recovery_model_include_top(effb4.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb4.model.input_shape = input_shape
+    effb4.model.to(_device)
     return effb4
 
 
@@ -395,25 +331,13 @@ def EfficientNetB5(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('17iTD12G9oW3jYAui84MKtdY4gjd9vpgG', dirname, 'efficientnet-b5.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b5.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb5.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb5.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb5.model = recovery_model
+    else:
+        effb5.model = _make_recovery_model_include_top(effb5.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb5.model.input_shape = input_shape
+    effb5.model.to(_device)
     return effb5
 
 
@@ -426,29 +350,17 @@ def EfficientNetB6(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1XJrKmcmMObN_nnjP2Z-YH_BQ3img58qF', dirname, 'efficientnet-b6.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b6.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb6.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb6.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb6.model = recovery_model
+    else:
+        effb6.model = _make_recovery_model_include_top(effb6.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb6.model.input_shape = input_shape
+    effb6.model.to(_device)
     return effb6
 
 
+
 def EfficientNetB7(include_top=True, pretrained=True, freeze_features=False,input_shape=(3, 600, 600), classes=1000, **kwargs):
     if input_shape is not None and len(input_shape) == 3:
         input_shape = tuple(input_shape)
@@ -458,23 +370,11 @@ def EfficientNetB7(include_top=True, pretrained=True, freeze_features=False,inpu
     if pretrained == True:
         download_model_from_google_drive('1M2DfvsNPRCWSo_CeXnUCQOR46rvOrhLl', dirname, 'efficientnet-b7.pth')
         recovery_model = fix_layer(load(os.path.join(dirname, 'efficientnet-b7.pth')))
-        recovery_model.input_shape = input_shape
-        recovery_model.to(_device)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            effb7.class_names = []
-        else:
-            if classes != 1000:
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-                effb7.class_names = []
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         effb7.model = recovery_model
+    else:
+        effb7.model = _make_recovery_model_include_top(effb7.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    effb7.model.input_shape = input_shape
+    effb7.model.to(_device)
     return effb7
