@@ -19,6 +19,7 @@ import torch.nn.functional as F
 from torch._six import container_abcs
 from torch.nn import init
 from torch.nn.parameter import Parameter
+from trident.models.pretrained_utils import _make_recovery_model_include_top
 
 from trident.backend.common import *
 from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, fix_layer, get_device,load
@@ -31,7 +32,7 @@ from trident.layers.pytorch_normalizations import get_normalization
 from trident.layers.pytorch_pooling import *
 from trident.optims.pytorch_trainer import *
 
-__all__ = ['basic_block','bottleneck', 'ResNet','ResNet50','ResNet101','ResNet152']
+__all__ = ['basic_block','bottleneck', 'ResNet','ResNet18','ResNet50','ResNet101','ResNet152','resnet','resnet18']
 
 _session = get_session()
 _device =get_device()
@@ -150,7 +151,69 @@ def ResNet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=
     #model.summary()
     return model
 
-#
+def resnet(block, layers, input_shape=(3, 224, 224), num_classes=1000, use_bias=False,  include_top=True, model_name='',
+           **kwargs):
+    """Instantiates the ResNet, ResNetV2, and ResNeXt architecture.
+
+    Args
+        block: a function that returns output tensor for the stacked residual blocks.
+        layers: list of integer, the number of  repeat units in each blocks.
+        input_shape: optional shape tuple, only to be specified
+            if `include_top` is False (otherwise the input shape
+            has to be`(3, 224, 224)` (with `channels_first` data format).
+            It should have exactly 3 inputs channels.
+        num_classes: optional number of classes to classify images
+            into, only to be specified if `include_top` is True, and
+            if no `weights` argument is specified.
+        use_bias: whether to use biases for convolutional layers or not
+            (True for ResNet and ResNetV2, False for ResNeXt).
+        include_top: whether to include the fully-connected layer at the top of the network.
+        model_name: string, model name.
+
+    Returns
+        A Keras model instance.
+
+    Raises
+        ValueError: in case of invalid argument for `weights`,  or invalid input shape.
+
+    """
+
+
+    def _make_layer(block, num_filters, blocklayers, strides=1, dilate=False,use_bias=use_bias,layer_name=''):
+        conv_shortcut=False
+        if strides!=1 or block is bottleneck:
+            conv_shortcut=True
+        layers = []
+        layers.append(block(num_filters=num_filters, strides=strides, expansion = 4, conv_shortcut=conv_shortcut,use_bias=use_bias, name=layer_name+'_0'))
+
+        for k in range(1, blocklayers):
+            layers.append(block(num_filters=num_filters,  strides=1, expansion = 4, conv_shortcut=False, use_bias=use_bias,name=layer_name+'_{0}'.format(k)))
+
+        laters_block=Sequential(*layers)
+        laters_block.name=layer_name
+        return laters_block
+
+    flow_list=[]
+    resnet = Sequential()
+    resnet.add_module('first_block',Conv2d_Block((7,7),64,strides=2,use_bias=use_bias,auto_pad=True,padding_mode='zero',normalization='batch',activation='relu',name='first_block'))
+    resnet.add_module('maxpool',(MaxPool2d((3,3),strides=2,auto_pad=True,padding_mode='zero')))
+    resnet.add_module('layer1',(_make_layer(block, 64, layers[0],strides=1, dilate=None,use_bias=use_bias,layer_name='layer1' )))
+    resnet.add_module('layer2',(_make_layer(block, 128, layers[1], strides=2, dilate=None,use_bias=use_bias,layer_name='layer2' )))
+    resnet.add_module('layer3',(_make_layer(block, 256, layers[2], strides=2, dilate=None,use_bias=use_bias,layer_name='layer3' )))
+    resnet.add_module('layer4' ,(_make_layer(block, 512, layers[3], strides=2, dilate=None,use_bias=use_bias,layer_name='layer4' )))
+    if include_top:
+        resnet.add_module('avg_pool', GlobalAvgPool2d(name='avg_pool'))
+        resnet.add_module('fc',Dense(num_classes,activation=None,name='fc'))
+        resnet.add_module('softmax', SoftMax(name='softmax'))
+    resnet.name=model_name
+    return resnet
+
+def resnet18(include_top=True,
+             input_shape=None,
+             classes=1000,
+             **kwargs):
+    return resnet(basic_block, [2, 2, 2, 2], input_shape=input_shape, include_top=include_top,model_name='resnet18')
+
 def ResNet18(include_top=True,
              pretrained=True,
              freeze_features=False,
@@ -167,25 +230,13 @@ def ResNet18(include_top=True,
         download_model_from_google_drive(model_urls['resnet18'], dirname, 'resnet18.pth')
         recovery_model = load(os.path.join(dirname, 'resnet18.pth'))
         recovery_model = fix_layer(recovery_model)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        recovery_model.name = 'resnet18'
-        recovery_model.eval()
-        recovery_model.to(_device)
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-        else:
-            if classes != 1000:
-                resnet18.class_names = []
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
         resnet18.model = recovery_model
+    else:
+        resnet18.model = _make_recovery_model_include_top(resnet18.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    resnet18.model.input_shape = input_shape
+    resnet18.model.to(_device)
     return resnet18
 
 def ResNet50(include_top=True,
@@ -203,26 +254,14 @@ def ResNet50(include_top=True,
         download_model_from_google_drive(model_urls['resnet50'],dirname,'resnet50.pth')
         recovery_model=load(os.path.join(dirname,'resnet50.pth'))
         recovery_model = fix_layer(recovery_model)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        recovery_model.name = 'resnet50'
-        recovery_model.eval()
-        recovery_model.to(_device)
-        if include_top==False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            resnet50.class_names = []
-        else:
-            if classes!=1000:
-                resnet50.class_names = []
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        resnet50.model = recovery_model
+    else:
+        resnet50.model = _make_recovery_model_include_top(resnet50.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    resnet50.model.input_shape = input_shape
+    resnet50.model.to(_device)
 
-        resnet50.model=recovery_model
     return resnet50
 
 def ResNet101(include_top=True,
@@ -240,25 +279,13 @@ def ResNet101(include_top=True,
         download_model_from_google_drive(model_urls['resnet101'],dirname,'resnet101.pth')
         recovery_model=load(os.path.join(dirname,'resnet101.pth'))
         recovery_model = fix_layer(recovery_model)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable=True
-        recovery_model.name = 'resnet101'
-        recovery_model.eval()
-        recovery_model.to(_device)
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            resnet101.class_names = []
-        else:
-            if classes != 1000:
-                resnet101.class_names = []
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-        resnet101.model=recovery_model
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        resnet101.model = recovery_model
+    else:
+        resnet101.model = _make_recovery_model_include_top(resnet101.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    resnet101.model.input_shape = input_shape
+    resnet101.model.to(_device)
     return resnet101
 
 
@@ -277,25 +304,13 @@ def ResNet152(include_top=True,
         download_model_from_google_drive(model_urls['resnet152'],dirname,'resnet152.pth')
         recovery_model=load(os.path.join(dirname,'resnet152.pth'))
         recovery_model = fix_layer(recovery_model)
-        if freeze_features:
-            recovery_model.trainable=False
-            recovery_model.fc.trainable = True
-        recovery_model.name = 'resnet152'
-        recovery_model.eval()
-        recovery_model.to(_device)
-        if include_top == False:
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            recovery_model.remove_at(-1)
-            resnet152.class_names = []
-        else:
-            if classes != 1000:
-                resnet152.class_names=[]
-                recovery_model.remove_at(-1)
-                recovery_model.remove_at(-1)
-                recovery_model.add_module('fc', Dense(classes, activation=None, name='fc'))
-                recovery_model.add_module('softmax', SoftMax())
-        resnet152.model=recovery_model
+        recovery_model = _make_recovery_model_include_top(recovery_model, include_top=include_top, classes=classes, freeze_features=freeze_features)
+        resnet152.model = recovery_model
+    else:
+        resnet152.model = _make_recovery_model_include_top(resnet152.model, include_top=include_top, classes=classes, freeze_features=False)
+
+    resnet152.model.input_shape = input_shape
+    resnet152.model.to(_device)
     return resnet152
 
 
