@@ -24,7 +24,7 @@ from torch.nn import init
 from torch.nn.parameter import Parameter
 
 from trident.backend.common import *
-from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, ModuleList, fix_layer, load
+from trident.backend.pytorch_backend import to_numpy, to_tensor, Layer, Sequential, ModuleList, fix_layer, load,get_device
 from trident.backend.pytorch_ops import *
 from trident.data.bbox_common import xywh2xyxy, xyxy2xywh
 from trident.data.image_common import *
@@ -40,7 +40,7 @@ from trident.models.pytorch_ssd import *
 __all__ = ['RfbNet', 'generate_priors']
 
 _session = get_session()
-_device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+_device = get_device()
 _epsilon = _session.epsilon
 _trident_dir = _session.trident_dir
 
@@ -239,9 +239,9 @@ class RFBnet(Layer):
         super(RFBnet, self).__init__(name=name)
         self.base_filters = base_filters
         backbond = tiny_mobile_rfbnet(self.base_filters)
-        self.backbond1 = Sequential(backbond[:8], name='backbond1')
-        self.backbond2 = Sequential(backbond[8:11], name='backbond2')
-        self.backbond3 = Sequential(backbond[11:13], name='backbond3')
+        self.backbond1 = Sequential(*backbond[:8], name='backbond1')
+        self.backbond2 = Sequential(*backbond[8:11], name='backbond2')
+        self.backbond3 = Sequential(*backbond[11:13], name='backbond3')
         self.prob_threshold = prob_threshold
         self.nms_threshold =nms_threshold
         self.variance = (center_variance, size_variance)
