@@ -17,7 +17,7 @@ import tensorflow as tf
 
 from trident.backend.common import *
 from trident.backend.tensorspec import *
-from trident.backend.tensorflow_backend import to_numpy, to_tensor, Layer, Sequential,load
+from trident.backend.tensorflow_backend import to_numpy, to_tensor, Layer, Sequential,load,get_device
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
 from trident.layers.tensorflow_activations import get_activation, Identity
@@ -132,13 +132,15 @@ def MobileNetV2(include_top=True,
         download_model_from_google_drive('15LtLJHpvimV6cFGqAwJ4QALNEjeATrKe',dirname,'mobilenet_v2_tf.pth')
         recovery_model=load(os.path.join(dirname,'mobilenet_v2_tf.pth'))
         recovery_model.eval()
-
-        if include_top==False:
-            recovery_model.remove_at(-1)
-        else:
-            if classes!=1000:
-                new_fc = Dense(classes, activation=None, name='fc')
-                new_fc.input_shape=recovery_model.fc.input_shape
-                recovery_model.fc=new_fc
-        mob.model=recovery_model
+        with tf.device(get_device()):
+            if include_top==False:
+                recovery_model.remove_at(-1)
+                recovery_model.remove_at(-1)
+                recovery_model.remove_at(-1)
+            else:
+                if classes!=1000:
+                    new_fc = Dense(classes, activation=None, name='fc')
+                    new_fc.input_shape=recovery_model.fc.input_shape
+                    recovery_model.fc=new_fc
+            mob.model=recovery_model
     return mob
\ No newline at end of file
