@@ -18,9 +18,10 @@ import tensorflow as tf
 import numpy as np
 from tensorflow.python.ops import  image_ops
 from trident.backend.common import *
+from trident.backend.tensorspec import *
 from trident.backend.tensorflow_backend import *
 from trident.backend.tensorflow_backend import to_numpy, to_tensor, Layer, Sequential, Combine
-from trident.backend.tensorflow_ops import meshgrid
+from trident.backend.tensorflow_ops import meshgrid,stack,where,expand_dims,concate
 from trident.data.bbox_common import clip_boxes_to_image, nms
 from trident.data.image_common import *
 from trident.data.utils import download_model_from_google_drive
