@@ -11,7 +11,7 @@ from tensorflow.python.ops import nn_ops
 from trident.backend.common import get_session,addindent,get_time_suffix,get_function,get_class,format_time,get_terminal_size,snake2camel,camel2snake
 from trident.backend.tensorflow_ops import *
 
-__all__ = ['accuracy','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
+__all__ = ['accuracy','pixel_accuracy','alpha_pixel_accuracy','iou','psnr','mean_absolute_error','mean_squared_error','mean_squared_logarithmic_error','mae','mse','rmse','msle','get_metric']
 
 
 def accuracy(output, target, topk=1, axis=-1, exclude_mask=False):
@@ -21,25 +21,22 @@ def accuracy(output, target, topk=1, axis=-1, exclude_mask=False):
     input_tensor = output.copy().detach()
     target_tensor = target.copy().detach()
 
-    input_tensor_exp = exp(input_tensor)
     is_logsoftmax = None
     from_logits = None
     output_exp = exp(input_tensor)
-    if (ndim(output) >= 1 and 'float' in str(output.dtype) and output.min() >= 0 and output.max() <= 1):
+    if (ndim(input_tensor) >= 1 and 'float' in str(input_tensor.dtype) and input_tensor.min() >= 0 and input_tensor.max() <= 1):
         is_logsoftmax = False
         from_logits = True
-        output = clip(output, min=1e-8, max=1 - 1e-8)
+        input_tensor = clip(input_tensor, min=1e-8, max=1 - 1e-8)
 
-    elif (ndim(output) >= 1 and 'float' in str(output.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
+    elif (ndim(output_exp) >= 1 and 'float' in str(output_exp.dtype) and output_exp.min() >= 0 and output_exp.max() <= 1):
         is_logsoftmax = True
         from_logits = True
-        output = clip(output, max=- 1e-8)
+        input_tensor = clip(output_exp, min=1e-8, max=1 - 1e-8)
     else:
         is_logsoftmax = False
         from_logits = False
 
-    if is_logsoftmax:
-        input_tensor = exp(input_tensor)
     if input_tensor.dtype != tf.int64 and topk == 1:
         if len(int_shape(input_tensor)) == 1:  # binary
             input_tensor =greater_equal(input_tensor,0.5)
@@ -61,6 +58,46 @@ def accuracy(output, target, topk=1, axis=-1, exclude_mask=False):
         return correct/batch_size
 
 
+def pixel_accuracy(output, target):
+    input_tensor = output.copy().detach()
+    target_tensor = target.copy().detach()
+    if input_tensor.dtype!=tf.int64 :
+        input_tensor=argmax(input_tensor,axis=-1).squeeze()
+    return equal(input_tensor,target_tensor).mean()
+
+def alpha_pixel_accuracy(output, alpha):
+    output_tensor = to_numpy(output)
+    alpha_tensor =  to_numpy(alpha)
+
+    trimap=alpha_tensor.copy()
+    trimap[(0<trimap)*(trimap<1)==True]=0.5
+    if len(output_tensor.shape)==len(alpha_tensor.shape)+1 and output_tensor.shape[1]==2:
+        # trimap_out=mask2trimap()(np.argmax(output_tensor,1))
+        # trimap_out1=trimap_out.copy()
+        # trimap_out1[trimap_out1==0.5]=0
+        # trimap_out2=trimap_out.copy()
+        # trimap_out2[trimap_out2 ==1] = 0
+        output_tensor=output_tensor[:,1,:,:]*np.argmax(output_tensor,1)#trimap_out*trimap_out1+output_tensor[:,1,:,:]*trimap_out2
+        output_tensor[output_tensor>0.95]=1
+    pixel_labeled = (output_tensor > 0).sum()
+    pixel_correct = ((output_tensor == alpha_tensor)*(trimap == 1)).sum()+ (np.less(np.abs(output_tensor - alpha_tensor),0.1).astype(np.float32)*(trimap == 0.5)).sum()
+    return pixel_correct/max(pixel_labeled,1)
+
+
+
+def iou(output, target):
+    input_tensor = output.copy().detach()
+    target_tensor = target.copy().detach()
+    if input_tensor.dtype != tf.int64:
+        input_tensor = argmax(input_tensor, axis=-1).squeeze()
+    if target_tensor.dtype != tf.int64:
+        target_tensor = argmax(target_tensor, axis=-1).squeeze()
+
+    intersection =( greater(input_tensor ,0) * equal(input_tensor ,target_tensor)).sum()
+    union=greater(( greater(input_tensor ,0) + greater(target_tensor ,0) ),0).sum()
+
+    return intersection/maximum(union,1)
+
 def psnr(output, target):
     if target.get_shape()!=output.get_shape() :
         raise ValueError('output shape {0} is not competable with target shape {1}'.format(output.shape,target.shape))
