@@ -14,21 +14,26 @@ from functools import partial
 import builtins
 
 import numpy as np
+from trident.callbacks.lr_schedulers import AdjustLRCallback
+
 from trident.backend import iteration_tools
 from trident.data.dataset import ZipDataset
-from trident.backend.common import to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, \
-    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path
+from trident.backend.common import get_backend,to_list, addindent, get_time_suffix, format_time, get_terminal_size, get_session, \
+    snake2camel, PrintException, unpack_singleton, enforce_singleton, OrderedDict, split_path, sanitize_path,make_dir_if_need
 from trident.backend.model import ModelBase, progress_bar
 from trident.callbacks.visualization_callbacks import *
 from trident.data.data_provider import *
 from trident.misc.ipython_utils import *
 from trident.misc.visualization_utils import tile_rgb_images, loss_metric_curve
 from trident.backend.tensorspec import TensorSpec, assert_spec_compatibility
-
+from trident.loggers.history import HistoryBase
 __all__ = ['TrainingPlan']
 
 _session = get_session()
-_backend = _session.backend
+_backend =get_backend()
+working_directory=_session.working_directory
+
+
 if _backend == 'pytorch':
     import torch
     import torch.nn as nn
@@ -49,6 +54,7 @@ class TrainingPlan(object):
         self._dataloaders = OrderedDict()
         self.num_epochs = 1
         self._minibatch_size = 1
+        self.steps=0
         self.warmup = 0
         self.default_collect_data_inteval = 1
         self.print_progress_frequency = 10
@@ -60,8 +66,9 @@ class TrainingPlan(object):
         self.save_model_frequency = -1
         self.save_model_unit = 'batch'
         self.execution_id = None
-
+        self.enable_tensorboard=False
         self._is_optimizer_warmup = False
+        self.summary_writer = None
 
         self.callbacks = []  # if self.callbacks is None:  #     self.callbacks = [  #
         # NumberOfEpochsStoppingCriterionCallback(1)]  # elif not any([issubclass(type(cb),
@@ -175,6 +182,7 @@ class TrainingPlan(object):
             training_item.name = 'model {0}'.format(n)
             self.training_names[n] = 'model {0}'.format(n)
         self.training_items[n] = training_item
+        training_item.training_context['training_name'] = self.training_names[n]
         self.training_items[n].start_epoch = start_epoch
         # backward compatibility
         for k, v in training_item.inputs.items():
@@ -207,6 +215,28 @@ class TrainingPlan(object):
         self.minibatch_size = minibatch_size
         return self
 
+    def within_tensorboard(self):
+        self.enable_tensorboard=True
+        #check weather have tensorboard
+        if get_backend() == 'pytorch':
+            try:
+                from trident.loggers.pytorch_tensorboard import SummaryWriter
+                self.summary_writer = SummaryWriter(os.path.join(working_directory, 'Logs'))
+
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        elif get_backend() == 'tensorflow':
+            try:
+                from trident.loggers.tensorflow_tensorboard import SummaryWriter
+                self.summary_writer = SummaryWriter(os.path.join(working_directory, 'Logs'))
+            except Exception as e:
+                print('Tensorboard initialize failed, please check the installation status about Tensorboard.')
+                print(e)
+                PrintException()
+        return self
+
     def out_sample_evaluation_scheduling(self, frequency: int, unit='batch', on_epoch_end=True):
         self.out_sample_evaluation_on_epoch_end = on_epoch_end
         self.out_sample_evaluation_frequency = frequency
@@ -217,6 +247,7 @@ class TrainingPlan(object):
 
         return self
 
+
     def print_progress_scheduling(self, frequency: int, unit='batch', on_epoch_end=True, show_loss_metric_curve=True):
         self.print_progress_on_epoch_end = on_epoch_end
         self.print_progress_frequency = frequency
@@ -314,9 +345,13 @@ class TrainingPlan(object):
                 for inp in v.signature.inputs.key_list:
                     data_feed[inp] = None
 
-            if 'x' in data_feed and 'input' in available_items:
-                data_feed['x'] = 'input'
-                available_items.remove('input')
+            if ( 'x' in data_feed  or  'input' in data_feed ) and 'input' in available_items:
+                if 'x' in data_feed :
+                    data_feed['x'] = 'input'
+                    available_items.remove('input')
+                elif  'input' in data_feed :
+                    data_feed['input'] = 'input'
+                    available_items.remove('input')
 
             data_symbols = iteration_tools.flatten([data_loader.traindata.data.symbol], iterable_types=(list, tuple))
             label_symbols = iteration_tools.flatten([data_loader.traindata.label.symbol], iterable_types=(list, tuple))
@@ -327,7 +362,7 @@ class TrainingPlan(object):
                 unpair_symbols.remove("")
 
             if len(trainingitem.signature.inputs) == len(data_symbols) == 1:
-                if assert_spec_compatibility(trainingitem.signature.inputs.value_list[0], data_loader.traindata.data.element_spec):
+                if trainingitem.signature.inputs.value_list[0].shape.is_compatible_with(data_loader.traindata.data.element_spec.shape):
                     data_feed[trainingitem.signature.inputs.key_list[0]] = data_loader.traindata.data.symbol
                     available_items.remove(data_loader.traindata.data.symbol)
             if len(trainingitem.signature.outputs) == len(label_symbols) == 1:
@@ -363,9 +398,27 @@ class TrainingPlan(object):
             exception_cnt = 0
             abnormal_num_count = 0
             # update callback
+            if self.enable_tensorboard:
+                for idx,(item, item_name) in enumerate(zip(self.training_items.value_list,self.training_names.value_list)):
+                    if hasattr(item,'training_context'):
+                        for context_item in list(item.training_context.values()):
+                            if isinstance(context_item,HistoryBase):
+                                context_item.enable_tensorboard=True
+                                context_item.training_name=item_name
+                                context_item.summary_writer=self.summary_writer
+                        item.training_context['training_name'] = item_name
+                        item.training_context['summary_writer']=self.summary_writer
+
+                make_dir_if_need(os.path.join(working_directory,'Logs'))
+
+                sys.stdout.writelines(['Please execute the command to initial tensorboard:  tensorboard --logdir={0}  --port 6006 \n\r'.format(os.path.join(working_directory,'Logs'))])
+                sys.stdout.writelines(['Tensorboard is initialized. You can access tensorboard at http://localhost:6006/   \n\r'])
+
             if not is_resume or only_steps == True:
                 max_name_length = builtins.max([len(name) for name in self.training_names.value_list])
                 for item in self.training_items.values():
+                    #sysnc device
+                    item.model.to(get_device())
                     item.training_context['execution_id'] = self.execution_id
                     item.training_context['max_name_length'] = max_name_length
                     for callback in self.callbacks:
@@ -395,16 +448,19 @@ class TrainingPlan(object):
             for epoch in range(self.num_epochs):
                 try:
                     for mbs, return_data in enumerate(data_loader):
+
                         if self.is_terminate:
                             for callback in self.callbacks:
                                 if callback.is_shared == True:
                                     callback.on_training_terminated(self.__dict__)
 
                             for k, trainitem in self.training_items.items():
+
                                 for callback in trainitem.training_context['callbacks']:
                                     if callback.is_shared == False:
                                         callback.on_training_terminated(trainitem.training_context)
                         else:
+
                             num_batches = len(data_loader.batch_sampler) * epoch + mbs
                             iter_data = OrderedDict()
                             if isinstance(return_data, OrderedDict):
@@ -456,20 +512,46 @@ class TrainingPlan(object):
                                 trainitem.training_context['model_name'] = trainitem_name
                                 if epoch < int(trainitem.start_epoch):
                                     trainitem.training_context['stop_update'] = 1
-                                trainitem.train_model(train_data, test_data, epoch if only_steps == False else 0,
+
+                                trainitem.train_model(train_data, test_data,
+                                                      epoch if only_steps == False else 0,
                                                       mbs if only_steps == False else num_batches,
-                                                      self.num_epochs if only_steps == False else 1, len(
-                                        data_loader.batch_sampler) if only_steps == False else max_batches,
+                                                      self.num_epochs if only_steps == False else 1,
+                                                      len(data_loader.batch_sampler) if only_steps == False else max_batches,
                                                       is_collect_data=mbs % collect_data_inteval == 0,
-                                                      is_print_batch_progress=self.print_progress_unit == 'batch' and mbs
-                                                                              % self.print_progress_frequency == 0,
-                                                      is_print_epoch_progress=self.print_progress_unit == 'epoch' and (
-                                                              epoch + 1) % self.print_progress_frequency == 0,
+                                                      is_print_batch_progress=self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0,
+                                                      is_print_epoch_progress=self.print_progress_unit == 'epoch' and (epoch + 1) % self.print_progress_frequency == 0,
                                                       log_gradients=keep_gradient_history, log_weights=keep_weights_history,
                                                       accumulate_grads=False, is_out_sample_evaluation=need_out_sample_evaluation)
-                            if self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0 and \
-                                    self.print_progress_unit == 'epoch' and (epoch + 1) % self.print_progress_frequency == 0:
-                                print('\n', flush=True)
+                            self.steps +=1
+
+
+                            if self.enable_tensorboard and len(self.training_items)>1 and mbs % collect_data_inteval == 0:
+                                compare_dict = OrderedDict()
+                                step=None
+                                for trainitem_name, trainitem in zip(self.training_names.value_list, self.training_items.value_list):
+                                    for k,v in trainitem.training_context["losses"].items():
+                                        if k not in compare_dict:
+                                            compare_dict[k]=OrderedDict()
+                                        compare_dict[k][k+"/"+trainitem_name]=v[-1][1]
+                                        step=v[-1][0]
+                                    for k,v in trainitem.training_context["metrics"].items():
+                                        if k not in compare_dict:
+                                            compare_dict[k]=OrderedDict()
+                                        compare_dict[k][k+"/"+trainitem_name]=v[-1][1]
+                                for k,v in compare_dict.items():
+                                    self.summary_writer.add_scalars(k,v,step)
+
+
+
+
+
+
+
+                            if (self.print_progress_unit == 'batch' and mbs % self.print_progress_frequency == 0) or \
+                                    (self.print_progress_unit == 'epoch' and (epoch + 1) % self.print_progress_frequency == 0):
+                                if len(self.training_items)>1:
+                                    print(' \n', flush=True)
 
                             for k, trainitem in self.training_items.items():
                                 for callback in trainitem.training_context['callbacks']:
@@ -483,6 +565,9 @@ class TrainingPlan(object):
                                     self.save_model_frequency == 0:
                                 for k, trainitem in self.training_items.items():
                                     trainitem.save_model(trainitem.training_context['save_path'])
+                                    # if self.enable_tensorboard:
+                                    #     trainitem.save_onnx(trainitem.training_context['save_path'].replace('.pth','.onnx'))
+                                    #     self.summary_writer.add_onnx_graph(trainitem.training_context['save_path'].replace('.pth','.onnx'));
 
                             if only_steps == True and num_batches >= max_batches - 1:
                                 for k, trainitem in self.training_items.items():
