@@ -1,4 +1,5 @@
-from  copy import deepcopy
+import copy
+from copy import deepcopy
 import math
 import random
 from itertools import count
@@ -8,16 +9,18 @@ from matplotlib.pylab import plt
 from torch import nn
 from typing import Any, List, Union, Mapping, Optional, Callable
 
+from trident.backend.tensorspec import TensorSpec, ObjectType
 from trident.misc.visualization_utils import loss_metric_curve
 
 from trident.data.image_common import image_backend_adaption
-from trident.backend.common import*
+from trident.backend.common import *
 from trident.backend.pytorch_ops import *
 from trident.backend.pytorch_backend import *
-from trident.optims.pytorch_optimizers import Optimizer,get_optimizer
+from trident.optims.pytorch_optimizers import Optimizer, get_optimizer
 from trident.optims.pytorch_trainer import Model
 from trident.optims.pytorch_losses import *
-from trident.reinforcement.utils import ReplayBuffer,Transition
+from trident.reinforcement.utils import ReplayBuffer, Transition
+
 import_or_install('gym')
 import gym
 
@@ -27,22 +30,26 @@ __all__ = ['PolicyBase', 'DqnPolicy']
 class PolicyBase(Model):
     """The base class for any RL policy.
     """
-    def __init__( self,network:Layer,env:gym.Env,memory_length:int=1000,name=None) -> None:
 
-        self.env=env
+    def __init__(self, network: Layer, env: gym.Env, memory_length: int = 1000, name=None) -> None:
+        super().__init__()
+        self.network = network
+        if name is not None:
+            self.network._name=name
+        self.env = env
         self.env.reset()
         self.observation_space = env.observation_space
         self.action_space = env.action_space
-        self.network=network
-        #self.agent_id = self.uuid
-        self.memory=ReplayBuffer(memory_length)
-        super().__init__(inputs=self.get_observation().repeat(2,0),output=deepcopy(network))
+       # self.agent_id = self.uuid
+        self.memory = ReplayBuffer(memory_length)
         self.name = name
+        self.setting_network()
 
-
+    def setting_network(self):
+        super()._initial_graph(input_shape=int_shape(self.get_observation())[1:],output=deepcopy(self.network))
 
     def get_observation(self):
-        return np.expand_dims(self.process_flow(self.env.render('rgb_array')),0)
+        return self.data_preprocess(self.env.render('rgb_array'))
 
     def select_action(self, state, **kwargs):
         pass
@@ -51,29 +58,56 @@ class PolicyBase(Model):
         observation_, reward, done, info = self.env.step(action.item())
         return reward
 
-    def experience_replay(self,batch_size):
-        train_data=OrderedDict()
+    def experience_replay(self, batch_size):
+        train_data = OrderedDict()
 
         return train_data
 
-    def process_flow(self,data):
-        if not hasattr(self,'transform_funcs') or self.transform_funcs is None:
-            self.transform_funcs=[]
-        if len(self.transform_funcs) == 0:
-            return image_backend_adaption(data)
-        if isinstance(data, np.ndarray):
-            for fc in self.transform_funcs:
-                data = fc(data)
-            data = image_backend_adaption(data)
-            return data
 
-
-    def learn(self,num_episodes = 3000,**kwargs):
+    def learn(self, num_episodes=3000, **kwargs):
         pass
 
-    def resume(self,num_episodes = 3000,**kwargs):
+    def resume(self, num_episodes=3000, **kwargs):
         pass
 
+    @property
+    def preprocess_flow(self):
+        return self._preprocess_flow
+
+    @preprocess_flow.setter
+    def preprocess_flow(self, value):
+        self._preprocess_flow = value
+        if isinstance(self.input_spec, TensorSpec):
+            self.input_spec = None
+        super()._initial_graph(inputs=to_tensor(self.get_observation()).repeat_elements(2, 0), output=deepcopy(self.network))
+        self.setting_network()
+
+        self.env.reset()
+    def data_preprocess(self, img_data):
+        if not hasattr(self,'_preprocess_flow') or self._preprocess_flow is None:
+            self._preprocess_flow=[]
+        if img_data.ndim==4:
+            return to_tensor(to_numpy([self.data_preprocess(im) for im in img_data]))
+        if len(self._preprocess_flow) == 0:
+            return image_backend_adaption(img_data)
+        if isinstance(img_data, np.ndarray):
+            for fc in self._preprocess_flow:
+                if self._model is not None and self.signature is not None and len(self.signature) > 1 and self.input_spec is not None:
+                    img_data = fc(img_data,spec=self.input_spec)
+                else:
+                    img_data = fc(img_data)
+            img_data = image_backend_adaption(img_data)
+            if self.input_spec is None :
+                self._model.input_spec= TensorSpec(shape=tensor_to_shape(to_tensor(img_data),need_exclude_batch_axis=False), object_type=ObjectType.rgb, name='input')
+
+                self.input_shape=self._model.input_spec.shape[1:]
+
+            return img_data
+        else:
+            return img_data
+
+
+
 class Dqn(PolicyBase):
     """The base class for any RL policy.
     """
@@ -81,12 +115,6 @@ class Dqn(PolicyBase):
     def __init__(self, network: Layer, env: gym.Env, memory_length: int = 100000, gamma=0.9, max_epsilon=0.9, min_epsilon=0.01, decay=200, target_update=10, batch_size=10,
                  name='dqn') -> None:
         super().__init__(network=network, env=env, memory_length=memory_length, name=name)
-        self.policy_net = self._model
-        self.policy_net.train()
-
-        self.target_net = deepcopy(self._model)
-        self.target_net.eval()
-        self.summary()
 
         self.gamma = gamma
         self.max_epsilon = max_epsilon
@@ -96,6 +124,16 @@ class Dqn(PolicyBase):
         self.batch_size = batch_size
         self.steps_done = 0
 
+    def setting_network(self):
+        super()._initial_graph(inputs=to_tensor(self.get_observation()).repeat_elements(2, 0), output=copy.deepcopy(self.network))
+
+        self.policy_net = self.model
+        self.policy_net.train()
+
+        self.target_net = deepcopy(self.network)
+        self.target_net.eval()
+        self.summary()
+
     def get_observation(self):
         # 在這邊維護取得STATE的方法
         return np.expand_dims(np.array(list(self.env.state)), 0)
@@ -122,7 +160,7 @@ class Dqn(PolicyBase):
         # Define the method how to get rewards
         # step to next time.
         observation_, reward, done, info = self.env.step(action)
-        return reward if not done else -10*done, done
+        return reward if not done else -10 * done, done
 
     def experience_replay(self, batch_size):
         # Experimenttal Replay
@@ -143,7 +181,7 @@ class Dqn(PolicyBase):
         q_next = max(next_q, axis=-1)
 
         # Calculate target rewards base on  Bellmann-equation.
-        #𝑄(𝑠,𝑎)=𝑟0+𝛾max𝑎𝑄∗(𝑠′,𝑎)
+        # 𝑄(𝑠,𝑎)=𝑟0+𝛾max𝑎𝑄∗(𝑠′,𝑎)
         target_rewards = reward_batch + (q_next * self.gamma) * greater(reward_batch, 0)
         target_rewards = target_rewards.detach()
 
@@ -197,12 +235,11 @@ class Dqn(PolicyBase):
                 next_state = self.get_observation()
 
                 # 將四元組儲存於記憶中，建議要減少「好案例」的儲存比例
-                if reward < 1 or (reward == 1 and i_episode < 20) or (
-                        reward == 1 and i_episode >= 20 and t < 100 and random.random() < 0.1 and i_episode >= 20 and t >= 100 and random.random() < 0.2):
-                    self.memory.push(state, action, next_state, reward)
+                #if reward<1.0 or (reward>=1.0 and i_episode<50 ) or (reward>=1.0 and i_episode>=50 and random.random()<0.5):
+                self.memory.push(state, action, next_state, reward)
 
                 # switch next t
-                state = deepcopy(next_state)
+                state =next_state
 
                 if start_train:
                     # get batch data from experimental replay
@@ -237,8 +274,7 @@ class Dqn(PolicyBase):
                             print('predict_rewards:', self.training_context['train_data']['predict_rewards'][:5])
                             print('target_rewards:', self.training_context['train_data']['target_rewards'][:5])
                             print('reward_batch:', self.training_context['train_data']['reward_batch'][:5])
-                            loss_metric_curve(self.epoch_loss_history, self.epoch_metric_history,
-                                              legend=['dqn'], calculate_base='epoch', imshow=imshow)
+                            loss_metric_curve(self.epoch_loss_history, self.epoch_metric_history,legend=['dqn'], calculate_base='epoch', imshow=imshow)
 
                     break
 
@@ -253,7 +289,9 @@ class Dqn(PolicyBase):
         plt.ioff()
         plt.show()
 
-DqnPolicy=Dqn
+
+DqnPolicy = Dqn
+
 
 class PolicyGradient(PolicyBase):
     """The base class for any RL policy.
@@ -262,8 +300,7 @@ class PolicyGradient(PolicyBase):
     def __init__(self, network: Layer, env: gym.Env, memory_length: int = 100000, gamma=0.9, max_epsilon=0.9, min_epsilon=0.01, decay=200, target_update=10, batch_size=10,
                  name='pg') -> None:
         super(PolicyGradient, self).__init__(network=network, env=env, memory_length=memory_length, name=name)
-        self.policy_net = self._model
-        self.policy_net.train()
+
 
         self.target_net = deepcopy(self._model)
         self.target_net.eval()
@@ -283,7 +320,6 @@ class PolicyGradient(PolicyBase):
 
     def select_action(self, state, **kwargs):
 
-
         sample = random.random()
         self.epsilon = self.min_epsilon + (self.max_epsilon - self.min_epsilon) * math.exp(-1.0 * self.steps_done / self.decay)
 
@@ -296,7 +332,7 @@ class PolicyGradient(PolicyBase):
             selected_action = np.random.randint(low=0, high=self.env.action_space.n)
             return selected_action
 
-    def discount_rewards(self,r:Tensor, gamma:float=0.999):
+    def discount_rewards(self, r: Tensor, gamma: float = 0.999):
         """使用1D rewards向量以及計算折價後獎賞 """
         discounted_r = np.zeros_like(r)
         running_add = 0
@@ -304,12 +340,12 @@ class PolicyGradient(PolicyBase):
             running_add = running_add * gamma + r[t]
             discounted_r[t] = running_add
         return discounted_r
-    
+
     def get_rewards(self, action):
         # Define the method how to get rewards
         # step to next time.
         observation_, reward, done, info = self.env.step(action)
-        return reward if not done else -10*done, done
+        return reward if not done else -10 * done, done
 
     def experience_replay(self, batch_size):
         # Experimenttal Replay
@@ -330,7 +366,7 @@ class PolicyGradient(PolicyBase):
         q_next = max(next_q, axis=-1)
 
         # Calculate target rewards base on  Bellmann-equation.
-        #𝑄(𝑠,𝑎)=𝑟0+𝛾max𝑎𝑄∗(𝑠′,𝑎)
+        # 𝑄(𝑠,𝑎)=𝑟0+𝛾max𝑎𝑄∗(𝑠′,𝑎)
         target_rewards = reward_batch + (q_next * self.gamma) * greater(reward_batch, 0)
         target_rewards = target_rewards.detach()
 
